@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -247,7 +247,7 @@ private:
     }
 public:
     // CONSTRUCTORS
-    DescopeVisitor(AstNetlist* nodep) {
+    explicit DescopeVisitor(AstNetlist* nodep) {
 	m_modp = NULL;
 	m_scopep = NULL;
 	m_needThis = false;
