@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2004-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2004-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -81,6 +81,9 @@ public:
 		    if (v3Global.dpi()) {
 			putMakeClassEntry(of, "verilated_dpi.cpp");
 		    }
+		    if (v3Global.opt.vpi()) {
+			putMakeClassEntry(of, "verilated_vpi.cpp");
+		    }
 		    if (v3Global.opt.savable()) {
 			putMakeClassEntry(of, "verilated_save.cpp");
 		    }
@@ -231,7 +234,7 @@ public:
     }
 
 public:
-    EmitMkVisitor(AstNetlist*) {
+    explicit EmitMkVisitor(AstNetlist*) {
 	emitClassMake();
 	emitOverallMake();
     }
