@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2004-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2004-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -36,7 +36,7 @@
 //######################################################################
 // Emit statements and math operators
 
-class EmitXmlFileVisitor : public EmitCBaseVisitor {
+class EmitXmlFileVisitor : public AstNVisitor {
     // MEMBERS
     V3OutFile*	m_ofp;
 
