@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -185,7 +185,7 @@ inline void V3FileDependImp::writeTimes(const string& filename, const string& cm
 }
 
 inline bool V3FileDependImp::checkTimes(const string& filename, const string& cmdlineIn) {
-    const auto_ptr<ifstream> ifp (V3File::new_ifstream_nodepend(filename));
+    const VL_UNIQUE_PTR<ifstream> ifp (V3File::new_ifstream_nodepend(filename));
     if (ifp->fail()) {
 	UINFO(2,"   --check-times failed: no input "<<filename<<endl);
 	return false;
@@ -363,6 +363,7 @@ private:
 	}
 	return out;
     }
+    // cppcheck-suppress unusedFunction unusedPrivateFunction
     string readFilterLine() {
 	// Slow, but we don't need it much
 	UINFO(9,"readFilterLine\n");
@@ -380,6 +381,7 @@ private:
 	UINFO(6,"filter-line-in: "<<line);
 	return line;
     }
+    // cppcheck-suppress unusedFunction unusedPrivateFunction
     void writeFilter(const string& out) {
 	if (debug()>=6) { UINFO(6,"filter-out: "<<out); if (out[out.length()-1]!='\n') cout<<endl; }
 	if (!m_pid) { v3error("--pipe-filter: write to closed file\n"); m_readEof = true; stop(); }
@@ -521,7 +523,7 @@ protected:
 	return out;
     }
     // CONSTRUCTORS
-    V3InFilterImp(const string& command) {
+    explicit V3InFilterImp(const string& command) {
 	m_readEof = false;
 	m_pid = 0;
 	m_pidExited = false;
@@ -553,6 +555,8 @@ V3OutFormatter::V3OutFormatter(const string& filename, V3OutFormatter::Language
     , m_lineno(1), m_column(0)
     , m_nobreak(false), m_prependIndent(true), m_indentLevel(0)
     , m_declSAlign(0), m_declNSAlign(0), m_declPadNum(0) {
+    m_blockIndent = v3Global.opt.decoration() ? 4 : 1;
+    m_commaWidth  = v3Global.opt.decoration() ? 50 : 150;
 }
 
 //----------------------------------------------------------------------
@@ -617,7 +621,7 @@ int V3OutFormatter::endLevels (const char *strg) {
 	// label/public/private:  Deindent by 2 spaces
 	const char* mp=cp;
 	for (; isalnum(*mp); mp++) ;
-	if (mp[0]==':' && mp[1]!=':') return (levels-INDBLK/2);
+	if (mp[0]==':' && mp[1]!=':') return (levels-m_blockIndent/2);
     }
 
     // We want "} else {" to be one level to the left of normal
@@ -625,16 +629,16 @@ int V3OutFormatter::endLevels (const char *strg) {
 	switch (*cp) {
 	case '}':
 	case ')':
-	    levels-=INDBLK;
+	    levels-=m_blockIndent;
 	    break;
 	case '<':
 	    if (m_lang==LA_XML) {
-		if (cp[1] == '/') levels-=INDBLK;
+		if (cp[1] == '/') levels-=m_blockIndent;
 	    }
 	    break;
 	case 'e':
 	    if (m_lang==LA_VERILOG && tokenEnd(cp)) {
-		levels-=INDBLK;
+		levels-=m_blockIndent;
 	    }
 	    break;
 	case '\t':
@@ -680,7 +684,11 @@ void V3OutFormatter::puts (const char *strg) {
 	    break;
 	case '(':
 	    indentInc();
-	    m_parenVec.push(m_column);
+	    if (v3Global.opt.decoration()) {
+		m_parenVec.push(m_column);  // Line up continuation with open paren, plus one indent
+	    } else {
+		m_parenVec.push(m_indentLevel*m_blockIndent); // Line up continuation with block+1
+	    }
 	    break;
 	case ')':
 	    if (!m_parenVec.empty()) m_parenVec.pop();
@@ -748,19 +756,19 @@ void V3OutFormatter::putBreak () {
     }
 }
 
-void V3OutFormatter::putsQuoted(const char* strg) {
+void V3OutFormatter::putsQuoted(const string& strg) {
     // Quote \ and " for use inside C programs
     // Don't use to quote a filename for #include - #include doesn't \ escape.
     putcNoTracking('"');
     string quoted = V3Number::quoteNameControls(strg);
-    for (const char* cp=quoted.c_str(); *cp; cp++) {
-	putcNoTracking (*cp);
+    for (string::const_iterator cp=quoted.begin(); cp!=quoted.end(); ++cp) {
+    	putcNoTracking (*cp);
     }
     putcNoTracking('"');
 }
-void V3OutFormatter::putsNoTracking (const char *strg) {
+void V3OutFormatter::putsNoTracking (const string& strg) {
     // Don't track {}'s, probably because it's a $display format string
-    for (const char* cp=strg; *cp; cp++) {
+    for (string::const_iterator cp=strg.begin(); cp!=strg.end(); ++cp) {
 	putcNoTracking (*cp);
     }
 }
@@ -789,7 +797,7 @@ void V3OutFormatter::putcNoTracking (char chr) {
     putcOutput (chr);
 }
 
-void V3OutFormatter::putAlign (bool/*AlignClass*/ isStatic, int align, int size, const char* prefix) {
+void V3OutFormatter::putAlign (bool/*AlignClass*/ isStatic, int align, int size, const string& prefix) {
     if (size==0) size=align;
     int alignSize = size; if (alignSize>8) alignSize=8;
     int& alignr = isStatic ? m_declSAlign : m_declNSAlign;
