@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -77,6 +77,7 @@ class V3InFilterImp;
 
 class V3InFilter {
     V3InFilterImp* m_impp;
+    V3InFilter(const V3InFilter&); ///< N/A, no copy constructor
 public:
     // TYPES
     typedef list<string> StrList;
@@ -86,7 +87,7 @@ public:
     bool readWholefile(const string& filename, StrList& outl);
 
     // CONSTRUCTORS
-    V3InFilter(const string& command);
+    explicit V3InFilter(const string& command);
     ~V3InFilter();
 };
 
@@ -96,8 +97,6 @@ public:
 class V3OutFormatter {
     // TYPES
     enum MiscConsts {
-	INDBLK = 4,	// Indentation per block level
-	WIDTH = 50,	// Width after which to break at ,'s
 	MAXSPACE = 80};	// After this indent, stop indenting more
 public:
     enum AlignClass {
@@ -114,6 +113,8 @@ private:
     // MEMBERS
     string	m_filename;
     Language	m_lang;		// Indenting Verilog code
+    int		m_blockIndent;	// Characters per block indent
+    int		m_commaWidth;	// Width after which to break at ,'s
     int		m_lineno;
     int		m_column;
     int		m_nobreak;	// Basic operator or begin paren, don't break next
@@ -132,29 +133,29 @@ public:
     V3OutFormatter(const string& filename, Language lang);
     virtual ~V3OutFormatter() {}
     // ACCESSORS
-    int  column() const { return m_column; }
+    int column() const { return m_column; }
+    int blockIndent() const { return m_blockIndent; }
+    void blockIndent(int flag) { m_blockIndent=flag; }
     // METHODS
     void printf(const char* fmt...) VL_ATTR_PRINTF(2);
     void puts(const char* strg);
     void puts(const string& strg) { puts(strg.c_str()); }
-    void putsNoTracking(const char* strg);
-    void putsNoTracking(const string& strg) { putsNoTracking(strg.c_str()); }
-    void putsQuoted(const char* strg);
-    void putsQuoted(const string& strg) { putsQuoted(strg.c_str()); }
+    void putsNoTracking(const string& strg);
+    void putsQuoted(const string& strg);
     void putBreak();  // Print linebreak if line is too wide
     void putBreakExpr();  // Print linebreak in expression if line is too wide
-    void putAlign(bool isstatic/*AlignClass*/, int align, int size=0/*=align*/, const char* prefix=""); // Declare a variable, with natural alignment
+    void putAlign(bool isstatic/*AlignClass*/, int align, int size=0/*=align*/, const string& prefix=""); // Declare a variable, with natural alignment
     void putbs(const char* strg) { putBreakExpr(); puts(strg); }
     void putbs(const string& strg) {  putBreakExpr(); puts(strg); }
-    bool exceededWidth() const { return m_column > WIDTH; }
+    bool exceededWidth() const { return m_column > m_commaWidth; }
     bool tokenStart(const char* cp, const char* cmp);
     bool tokenEnd(const char* cp);
-    void indentInc() { m_indentLevel += INDBLK; }
+    void indentInc() { m_indentLevel += m_blockIndent; }
     void indentDec() {
-	m_indentLevel -= INDBLK;
+	m_indentLevel -= m_blockIndent;
 	UASSERT(m_indentLevel>=0, ": "<<m_filename<<": Underflow of indentation\n");
     }
-    void blockInc() { m_parenVec.push(m_indentLevel + INDBLK); }
+    void blockInc() { m_parenVec.push(m_indentLevel + m_blockIndent); }
     void blockDec() { if (!m_parenVec.empty()) m_parenVec.pop(); }
     // STATIC METHODS
     static const string indentSpaces(int levels);
@@ -183,7 +184,7 @@ private:
 class V3OutCFile : public V3OutFile {
     int		m_private;
 public:
-    V3OutCFile(const string& filename) : V3OutFile(filename, V3OutFormatter::LA_C) {
+    explicit V3OutCFile(const string& filename) : V3OutFile(filename, V3OutFormatter::LA_C) {
 	resetPrivate();
     }
     virtual ~V3OutCFile() {}
@@ -208,7 +209,7 @@ public:
 
 class V3OutScFile : public V3OutCFile {
 public:
-    V3OutScFile(const string& filename) : V3OutCFile(filename) {}
+    explicit V3OutScFile(const string& filename) : V3OutCFile(filename) {}
     virtual ~V3OutScFile() {}
     virtual void putsHeader() { puts("// Verilated -*- SystemC -*-\n"); }
     virtual void putsIntTopInclude() {
@@ -219,7 +220,7 @@ public:
 
 class V3OutSpFile : public V3OutCFile {
 public:
-    V3OutSpFile(const string& filename) : V3OutCFile(filename) {}
+    explicit V3OutSpFile(const string& filename) : V3OutCFile(filename) {}
     virtual ~V3OutSpFile() {}
     virtual void putsHeader() { puts("// Verilated -*- SystemC -*-\n"); }
     virtual void putsIntTopInclude() {
@@ -230,21 +231,23 @@ public:
 
 class V3OutVFile : public V3OutFile {
 public:
-    V3OutVFile(const string& filename) : V3OutFile(filename, V3OutFormatter::LA_VERILOG) {}
+    explicit V3OutVFile(const string& filename) : V3OutFile(filename, V3OutFormatter::LA_VERILOG) {}
     virtual ~V3OutVFile() {}
     virtual void putsHeader() { puts("// Verilated -*- Verilog -*-\n"); }
 };
 
 class V3OutXmlFile : public V3OutFile {
 public:
-    V3OutXmlFile(const string& filename) : V3OutFile(filename, V3OutFormatter::LA_XML) {}
+    explicit V3OutXmlFile(const string& filename) : V3OutFile(filename, V3OutFormatter::LA_XML) {
+	blockIndent(2);
+    }
     virtual ~V3OutXmlFile() {}
     virtual void putsHeader() { puts("<?xml version=\"1.0\" ?>\n"); }
 };
 
 class V3OutMkFile : public V3OutFile {
 public:
-    V3OutMkFile(const string& filename) : V3OutFile(filename, V3OutFormatter::LA_MK) {}
+    explicit V3OutMkFile(const string& filename) : V3OutFile(filename, V3OutFormatter::LA_MK) {}
     virtual ~V3OutMkFile() {}
     virtual void putsHeader() { puts("# Verilated -*- Makefile -*-\n"); }
     // No automatic indentation yet.
