@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -110,7 +110,7 @@ void FileLine::lineDirective(const char* textp, int& enterExitRef) {
     const char *ln = textp;
     while (*textp && !isspace(*textp)) textp++;
     if (isdigit(*ln)) {
-	this->lineno(atoi(ln));
+	lineno(atoi(ln));
     }
     while (*textp && (isspace(*textp) || *textp=='"')) textp++;
 
@@ -120,7 +120,7 @@ void FileLine::lineDirective(const char* textp, int& enterExitRef) {
     if (textp != fn) {
 	string strfn = fn;
 	strfn = strfn.substr(0, textp-fn);
-	this->filename(strfn);
+	filename(strfn);
     }
 
     // Grab level
@@ -229,13 +229,13 @@ void FileLine::modifyStateInherit(const FileLine* fromp) {
     for (int codei=V3ErrorCode::EC_MIN; codei<V3ErrorCode::_ENUM_MAX; codei++) {
 	V3ErrorCode code = (V3ErrorCode)codei;
 	if (fromp->warnIsOff(code)) {
-	    this->warnOff(code, true);
+	    warnOff(code, true);
 	}
     }
 }
 
 void FileLine::v3errorEnd(ostringstream& str) {
-    if (this && m_lineno) {
+    if (m_lineno) {
 	ostringstream nsstr;
 	nsstr<<this<<str.str();
 	if (warnIsOff(V3Error::errorCode())) V3Error::suppressThisWarning();
@@ -246,7 +246,7 @@ void FileLine::v3errorEnd(ostringstream& str) {
 }
 
 string FileLine::warnMore() const {
-    if (this && m_lineno) {
+    if (m_lineno) {
 	return V3Error::warnMore()+ascii()+": ";
     } else {
 	return V3Error::warnMore();
