@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -46,6 +46,8 @@
 
 typedef list<AstNodeVarRef*> GateVarRefList;
 
+#define GATE_DEDUP_MAX_DEPTH  20
+
 //######################################################################
 
 class GateBaseVisitor : public AstNVisitor {
@@ -496,7 +498,7 @@ private:
 
 public:
     // CONSTUCTORS
-    GateVisitor(AstNode* nodep) {
+    explicit GateVisitor(AstNode* nodep) {
 	AstNode::user1ClearTree();
 	m_logicVertexp = NULL;
 	m_scopep = NULL;
@@ -617,7 +619,7 @@ void GateVisitor::optimizeSignals(bool allowMultiIn) {
 				    varvertexp->propagateAttrClocksFrom(vvertexp);
 				}
 				// Remove the edge
-				edgep->unlinkDelete(); edgep=NULL;
+				edgep->unlinkDelete(); VL_DANGLING(edgep);
 				++m_statRefs;
 				edgep = vvertexp->outBeginp();
 			    }
@@ -625,7 +627,7 @@ void GateVisitor::optimizeSignals(bool allowMultiIn) {
 			if (removedAllUsages) {
 			    // Remove input links
 			    while (V3GraphEdge* edgep = vvertexp->inBeginp()) {
-				edgep->unlinkDelete(); edgep=NULL;
+				edgep->unlinkDelete(); VL_DANGLING(edgep);
 			    }
 			    // Clone tree so we remember it for tracing, and keep the pointer
 			    // to the "ALWAYS" part of the tree as part of this statement
@@ -678,25 +680,25 @@ void GateVisitor::replaceAssigns() {
 	    if (vscp->valuep() && !vscp->valuep()->castNodeMath()) {
 		//if (debug()>9) vscp->dumpTree(cout, "-vscPre:  ");
 		while (AstNode* delp=vscp->valuep()->castComment()) {
-		    delp->unlinkFrBack()->deleteTree(); delp=NULL;
+		    delp->unlinkFrBack()->deleteTree(); VL_DANGLING(delp);
 		}
 		if (AstInitial* delp=vscp->valuep()->castInitial()) {
 		    AstNode* bodyp=delp->bodysp();
 		    bodyp->unlinkFrBackWithNext();
 		    delp->replaceWith(bodyp);
-		    delp->deleteTree(); delp=NULL;
+		    delp->deleteTree(); VL_DANGLING(delp);
 		}
 		if (AstAlways* delp=vscp->valuep()->castAlways()) {
 		    AstNode* bodyp=delp->bodysp();
 		    bodyp->unlinkFrBackWithNext();
 		    delp->replaceWith(bodyp);
-		    delp->deleteTree(); delp=NULL;
+		    delp->deleteTree(); VL_DANGLING(delp);
 		}
 		if (AstNodeAssign* delp=vscp->valuep()->castNodeAssign()) {
 		    AstNode* rhsp=delp->rhsp();
 		    rhsp->unlinkFrBack();
 		    delp->replaceWith(rhsp);
-		    delp->deleteTree(); delp=NULL;
+		    delp->deleteTree(); VL_DANGLING(delp);
 		}
 		//if (debug()>9) {vscp->dumpTree(cout, "-vscDone: "); cout<<endl;}
 		if (!vscp->valuep()->castNodeMath()
@@ -749,7 +751,7 @@ void GateVisitor::consumedMove() {
 		// Eventually: Move the statement to a new active block with "tracing-on" sensitivity
 		UINFO(8,"    Remove unconsumed "<<nodep<<endl);
 		nodep->unlinkFrBack();
-		pushDeletep(nodep); nodep=NULL;
+		pushDeletep(nodep); VL_DANGLING(nodep);
 	    }
 	}
     }
@@ -817,7 +819,7 @@ private:
 	    // Make the substp an rvalue like nodep. This facilitate the hashing in dedupe.
 	    if (AstNodeVarRef* varrefp = substp->castNodeVarRef()) varrefp->lvalue(false);
 	    nodep->replaceWith(substp);
-	    nodep->deleteTree(); nodep=NULL;
+	    nodep->deleteTree(); VL_DANGLING(nodep);
 	}
     }
     virtual void visit(AstNode* nodep, AstNUser*) {
@@ -1008,52 +1010,58 @@ private:
     // AstUser2InUse		m_inuser2;	(Allocated for use in GateVisitor)
     V3Double0			m_numDeduped;	// Statistic tracking
     GateDedupeVarVisitor	m_varVisitor;	// Looks for a dupe of the logic
+    int				m_depth;	// Iteration depth
 
-    virtual AstNUser* visit(GateVarVertex *vvertexp, AstNUser*) {
+    virtual AstNUser* visit(GateVarVertex* vvertexp, AstNUser*) {
 	// Check that we haven't been here before
+	if (m_depth > GATE_DEDUP_MAX_DEPTH) return NULL;  // Break loops; before user2 set so hit this vertex later
 	if (vvertexp->varScp()->user2()) return NULL;
 	vvertexp->varScp()->user2(true);
 
-	AstNodeVarRef* dupVarRefp = (AstNodeVarRef*) vvertexp->iterateInEdges(*this, (AstNUser*) vvertexp);
+	m_depth++;
+	if (vvertexp->inSize1()) {
+	    AstNodeVarRef* dupVarRefp = (AstNodeVarRef*) vvertexp->iterateInEdges(*this, (AstNUser*) vvertexp);
 
-	if (dupVarRefp && vvertexp->inSize1()) {
-	    V3GraphEdge* edgep = vvertexp->inBeginp();
-	    GateLogicVertex* lvertexp = (GateLogicVertex*)edgep->fromp();
-	    if (!vvertexp->dedupable()) vvertexp->varScp()->v3fatalSrc("GateLogicVertex* visit should have returned NULL if consumer var vertex is not dedupable.");
-	    GateOkVisitor okVisitor(lvertexp->nodep(), false, true);
-	    if (okVisitor.isSimple()) {
-		AstVarScope* dupVarScopep = dupVarRefp->varScopep();
-		GateVarVertex* dupVvertexp = (GateVarVertex*) (dupVarScopep->user1p());
-		UINFO(4,"replacing " << vvertexp << " with " << dupVvertexp << endl);
-		++m_numDeduped;
-		// Replace all of this varvertex's consumers with dupVarRefp
-		for (V3GraphEdge* outedgep = vvertexp->outBeginp();outedgep;) {
-		    GateLogicVertex* consumeVertexp = dynamic_cast<GateLogicVertex*>(outedgep->top());
-		    AstNode* consumerp = consumeVertexp->nodep();
-		    GateElimVisitor elimVisitor(consumerp,vvertexp->varScp(),dupVarRefp);
-		    outedgep = outedgep->relinkFromp(dupVvertexp);
+	    if (dupVarRefp) {
+		V3GraphEdge* edgep = vvertexp->inBeginp();
+		GateLogicVertex* lvertexp = (GateLogicVertex*)edgep->fromp();
+		if (!vvertexp->dedupable()) vvertexp->varScp()->v3fatalSrc("GateLogicVertex* visit should have returned NULL if consumer var vertex is not dedupable.");
+		GateOkVisitor okVisitor(lvertexp->nodep(), false, true);
+		if (okVisitor.isSimple()) {
+		    AstVarScope* dupVarScopep = dupVarRefp->varScopep();
+		    GateVarVertex* dupVvertexp = (GateVarVertex*) (dupVarScopep->user1p());
+		    UINFO(4,"replacing " << vvertexp << " with " << dupVvertexp << endl);
+		    ++m_numDeduped;
+		    // Replace all of this varvertex's consumers with dupVarRefp
+		    for (V3GraphEdge* outedgep = vvertexp->outBeginp(); outedgep; ) {
+			GateLogicVertex* consumeVertexp = dynamic_cast<GateLogicVertex*>(outedgep->top());
+			AstNode* consumerp = consumeVertexp->nodep();
+			GateElimVisitor elimVisitor(consumerp,vvertexp->varScp(),dupVarRefp);
+			outedgep = outedgep->relinkFromp(dupVvertexp);
+		    }
+
+		    // Propagate attributes
+		    dupVvertexp->propagateAttrClocksFrom(vvertexp);
+		    // Remove inputs links
+		    while (V3GraphEdge* inedgep = vvertexp->inBeginp()) {
+			inedgep->unlinkDelete(); VL_DANGLING(inedgep);
+		    }
+		    // replaceAssigns() does the deleteTree on lvertexNodep in a later step
+		    AstNode* lvertexNodep = lvertexp->nodep();
+		    lvertexNodep->unlinkFrBack();
+		    vvertexp->varScp()->valuep(lvertexNodep);
+		    lvertexNodep = NULL;
+		    vvertexp->user(true);
+		    lvertexp->user(true);
 		}
-
-		// Propogate attributes
-		dupVvertexp->propagateAttrClocksFrom(vvertexp);
-
-		// Remove inputs links
-		while (V3GraphEdge* inedgep = vvertexp->inBeginp()) {
-		    inedgep->unlinkDelete(); inedgep=NULL;
-		}
-		// replaceAssigns() does the deleteTree on lvertexNodep in a later step
-		AstNode* lvertexNodep = lvertexp->nodep();
-		lvertexNodep->unlinkFrBack();
-		vvertexp->varScp()->valuep(lvertexNodep);
-		lvertexNodep = NULL;
-		vvertexp->user(true);
-		lvertexp->user(true);
 	    }
 	}
+	m_depth--;
 	return NULL;
     }
 
-    // Returns a varref that has the same logic input
+    // Given iterated logic, starting at vup which was consumer's GateVarVertex
+    // Returns a varref that has the same logic input; or NULL if none
     virtual AstNUser* visit(GateLogicVertex* lvertexp, AstNUser* vup) {
 	lvertexp->iterateInEdges(*this);
 
@@ -1072,7 +1080,9 @@ private:
     }
 
 public:
-    GateDedupeGraphVisitor() {}
+    GateDedupeGraphVisitor() {
+	m_depth = 0;
+    }
     void dedupeTree(GateVarVertex* vvertexp) {
 	vvertexp->accept(*this);
     }
@@ -1161,14 +1171,14 @@ private:
 			if (AstSel* newselp = merge(preselp, curselp)) {
 			    UINFO(5, "assemble to new sel: "<<newselp<<endl);
 			    // replace preSel with newSel
-			    preselp->replaceWith(newselp); preselp->deleteTree(); preselp = NULL;
+			    preselp->replaceWith(newselp); preselp->deleteTree(); VL_DANGLING(preselp);
 			    // create new rhs for pre assignment
 			    AstNode* newrhsp = new AstConcat(m_assignp->rhsp()->fileline(), m_assignp->rhsp()->cloneTree(false), assignp->rhsp()->cloneTree(false));
 			    AstNode* oldrhsp = m_assignp->rhsp();
-			    oldrhsp->replaceWith(newrhsp); oldrhsp->deleteTree(); oldrhsp = NULL;
+			    oldrhsp->replaceWith(newrhsp); oldrhsp->deleteTree(); VL_DANGLING(oldrhsp);
 			    m_assignp->dtypeChgWidthSigned(m_assignp->width()+assignp->width(), m_assignp->width()+assignp->width(), AstNumeric::fromBool(true));
 			    // don't need to delete, will be handled
-			    //assignp->unlinkFrBack(); assignp->deleteTree(); assignp = NULL;
+			    //assignp->unlinkFrBack(); assignp->deleteTree(); VL_DANGLING(assignp);
 
 			    // update the graph
 			    {
@@ -1179,11 +1189,11 @@ private:
 					ledgep = ledgep->inNextp();
 					GateEitherVertex* fromvp = dynamic_cast<GateEitherVertex*>(oedgep->fromp());
 					new V3GraphEdge(m_graphp, fromvp, m_logicvp, 1);
-					oedgep->unlinkDelete(); oedgep = NULL;
+					oedgep->unlinkDelete(); VL_DANGLING(oedgep);
 				    }
 				}
 				// delete all outedges to lvertexp, only one
-				oldedgep->unlinkDelete(); oldedgep = NULL;
+				oldedgep->unlinkDelete(); VL_DANGLING(oldedgep);
 			    }
 			    ++m_numMergedAssigns;
 			} else {
@@ -1202,7 +1212,7 @@ private:
     }
 
 public:
-    GateMergeAssignsGraphVisitor(V3Graph* graphp) {
+    explicit GateMergeAssignsGraphVisitor(V3Graph* graphp) {
 	m_assignp = NULL;
 	m_activep = NULL;
 	m_logicvp = NULL;
@@ -1241,7 +1251,7 @@ private:
 	    AstNode* valuep = assp->rhsp();
 	    valuep->unlinkFrBack();
 	    assp->replaceWith(valuep);
-	    assp->deleteTree(); assp=NULL;
+	    assp->deleteTree(); VL_DANGLING(assp);
 	}
     }
     // Speedups
@@ -1253,7 +1263,7 @@ private:
 
 public:
     // CONSTUCTORS
-    GateDeassignVisitor(AstNode* nodep) {
+    explicit GateDeassignVisitor(AstNode* nodep) {
 	nodep->accept(*this);
     }
     virtual ~GateDeassignVisitor() {}
