@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -109,7 +109,7 @@ private:
 		AstVarScope* newvscp = genInpClk(vscp);
 		AstVarRef* newrefp = new AstVarRef(nodep->fileline(), newvscp, nodep->lvalue());
 		nodep->replaceWith(newrefp);
-		pushDeletep(nodep); nodep=NULL;
+		pushDeletep(nodep); VL_DANGLING(nodep);
 	    }
 	}
     }
@@ -213,7 +213,7 @@ private:
     }
 public:
     // CONSTRUCTORS
-    GenClkReadVisitor(AstNetlist* nodep) {
+    explicit GenClkReadVisitor(AstNetlist* nodep) {
 	m_activep = NULL;
 	m_assignp = NULL;
 	m_topModp = NULL;
