@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -269,7 +269,7 @@ void GraphAcyc::deleteMarked () {
 	nextp = vertexp->verticesNextp();
 	GraphAcycVertex* avertexp = (GraphAcycVertex*)vertexp;
 	if (avertexp->isDelete()) {
-	    avertexp->unlinkDelete(&m_breakGraph); avertexp=NULL;
+	    avertexp->unlinkDelete(&m_breakGraph); VL_DANGLING(avertexp);
 	}
     }
 }
@@ -285,13 +285,13 @@ void GraphAcyc::simplifyNone (GraphAcycVertex* avertexp) {
 	while (V3GraphEdge* edgep = avertexp->outBeginp()) {
 	    V3GraphVertex* otherVertexp = edgep->top();
 	    //UINFO(9,"  out "<<otherVertexp<<endl);
-	    edgep->unlinkDelete(); edgep = NULL;
+	    edgep->unlinkDelete(); VL_DANGLING(edgep);
 	    workPush(otherVertexp);
 	}
 	while (V3GraphEdge* edgep = avertexp->inBeginp()) {
 	    V3GraphVertex* otherVertexp = edgep->fromp();
 	    //UINFO(9,"  in  "<<otherVertexp<<endl);
-	    edgep->unlinkDelete(); edgep = NULL;
+	    edgep->unlinkDelete(); VL_DANGLING(edgep);
 	    workPush(otherVertexp);
 	}
     }
@@ -319,10 +319,11 @@ void GraphAcyc::simplifyOne (GraphAcycVertex* avertexp) {
 					    && (!outEdgep->cutable()
 						|| inEdgep->weight()<outEdgep->weight() ))
 					   ? inEdgep : outEdgep);
+	    // cppcheck-suppress leakReturnValNotUsed
 	    edgeFromEdge(templateEdgep, inVertexp, outVertexp);
 	    // Remove old edge
-	    inEdgep->unlinkDelete(); inEdgep = NULL;
-	    outEdgep->unlinkDelete(); outEdgep = NULL; templateEdgep=NULL;
+	    inEdgep->unlinkDelete(); VL_DANGLING(inEdgep);
+	    outEdgep->unlinkDelete(); VL_DANGLING(outEdgep); VL_DANGLING(templateEdgep);
 	    workPush(inVertexp);
 	    workPush(outVertexp);
 	}
@@ -353,12 +354,13 @@ void GraphAcyc::simplifyOut (GraphAcycVertex* avertexp) {
 		    return;
 		}
 		// Make a new edge connecting the two vertices directly
+		// cppcheck-suppress leakReturnValNotUsed
 		edgeFromEdge(inEdgep, inVertexp, outVertexp);
 		// Remove old edge
-		inEdgep->unlinkDelete(); inEdgep = NULL;
+		inEdgep->unlinkDelete(); VL_DANGLING(inEdgep);
 		workPush(inVertexp);
 	    }
-	    outEdgep->unlinkDelete(); outEdgep = NULL;
+	    outEdgep->unlinkDelete(); VL_DANGLING(outEdgep);
 	    workPush(outVertexp);
 	}
     }
@@ -381,18 +383,18 @@ void GraphAcyc::simplifyDup (GraphAcycVertex* avertexp) {
 		// !cutable duplicates prev !cutable: we can ignore it, redundant
 		//  cutable duplicates prev !cutable: know it's not a relevant loop, ignore it
 		UINFO(8,"    DelDupEdge "<<avertexp<<" -> "<<edgep->top()<<endl);
-		edgep->unlinkDelete(); edgep = NULL;
+		edgep->unlinkDelete(); VL_DANGLING(edgep);
 	    } else if (!edgep->cutable()) {
 		// !cutable duplicates prev  cutable: delete the earlier cutable
 		UINFO(8,"    DelDupPrev "<<avertexp<<" -> "<<prevEdgep->top()<<endl);
-		prevEdgep->unlinkDelete(); prevEdgep = NULL;
+		prevEdgep->unlinkDelete(); VL_DANGLING(prevEdgep);
 		outVertexp->userp(edgep);
 	    } else {
 		//  cutable duplicates prev  cutable: combine weights
 		UINFO(8,"    DelDupComb "<<avertexp<<" -> "<<edgep->top()<<endl);
 		prevEdgep->weight (prevEdgep->weight() + edgep->weight());
 		addOrigEdgep (prevEdgep, edgep);
-		edgep->unlinkDelete(); edgep = NULL;
+		edgep->unlinkDelete(); VL_DANGLING(edgep);
 	    }
 	    workPush(outVertexp);
 	    workPush(avertexp);
@@ -410,7 +412,7 @@ void GraphAcyc::cutBasic (GraphAcycVertex* avertexp) {
 	nextp = edgep->outNextp();
 	if (edgep->cutable() && edgep->top()==avertexp) {
 	    cutOrigEdge (edgep, "  Cut Basic");
-	    edgep->unlinkDelete(); edgep = NULL;
+	    edgep->unlinkDelete(); VL_DANGLING(edgep);
 	    workPush(avertexp);
 	}
     }
@@ -431,7 +433,7 @@ void GraphAcyc::cutBackward (GraphAcycVertex* avertexp) {
 	nextp = edgep->outNextp();
 	if (edgep->cutable() && edgep->top()->user()) {
 	    cutOrigEdge (edgep, "  Cut A->B->A");
-	    edgep->unlinkDelete(); edgep = NULL;
+	    edgep->unlinkDelete(); VL_DANGLING(edgep);
 	    workPush(avertexp);
 	}
     }
@@ -495,7 +497,7 @@ void GraphAcyc::placeTryEdge(V3GraphEdge* edgep) {
 	// Adding this edge would cause a loop, kill it
 	edgep->cutable(true);  // So graph still looks pretty
 	cutOrigEdge (edgep, "  Cut loop");
-	edgep->unlinkDelete(); edgep = NULL;
+	edgep->unlinkDelete(); VL_DANGLING(edgep);
 	// Backout the ranks we calculated
 	while (GraphAcycVertex* vertexp = workBeginp()) {
 	    workPop();
