@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -109,10 +109,10 @@ private:
 		    }
 		    if (saveOld) {
 			if (m_sumWeights) prevEdgep->weight(prevEdgep->weight() + edgep->weight());
-			edgep->unlinkDelete(); edgep = NULL;
+			edgep->unlinkDelete(); VL_DANGLING(edgep);
 		    } else {
 			if (m_sumWeights) edgep->weight(prevEdgep->weight() + edgep->weight());
-			prevEdgep->unlinkDelete(); prevEdgep = NULL;
+			prevEdgep->unlinkDelete(); VL_DANGLING(prevEdgep);
 			outVertexp->userp(edgep);
 		    }
 		}
