@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -135,14 +135,14 @@ private:
 	    } else {
 		m_modp->user1(1);
 	    }
-	    nodep->unlinkFrBack()->deleteTree(); nodep=NULL;  // Remove so don't propagate to upper cell...
+	    nodep->unlinkFrBack()->deleteTree(); VL_DANGLING(nodep);  // Remove so don't propagate to upper cell...
 	} else if (nodep->pragType() == AstPragmaType::NO_INLINE_MODULE) {
 	    if (!m_modp) {
 		nodep->v3error("Inline pragma not under a module");
 	    } else {
 		cantInline("Pragma NO_INLINE_MODULE",false);
 	    }
-	    nodep->unlinkFrBack()->deleteTree(); nodep=NULL;  // Remove so don't propagate to upper cell...
+	    nodep->unlinkFrBack()->deleteTree(); VL_DANGLING(nodep);  // Remove so don't propagate to upper cell...
 	} else {
 	    nodep->iterateChildren(*this);
 	}
@@ -151,14 +151,6 @@ private:
 	// Cleanup link until V3LinkDot can correct it
 	nodep->varp(NULL);
     }
-    virtual void visit(AstVar* nodep, AstNUser*) {
-	// Can't look at AstIfaceRefDType directly as it is no longer underneath the module
-	if (nodep->isIfaceRef()) {
-	    // Unsupported: Inlining of modules with ifaces (see AstIface comment above)
-	    if (m_modp) cantInline("Interfaced",true);
-	}
-	nodep->iterateChildren(*this);
-    }
     virtual void visit(AstNodeFTaskRef* nodep, AstNUser*) {
 	// Cleanup link until V3LinkDot can correct it
 	if (!nodep->packagep()) nodep->taskp(NULL);
@@ -185,7 +177,7 @@ private:
 
 public:
     // CONSTUCTORS
-    InlineMarkVisitor(AstNode* nodep) {
+    explicit InlineMarkVisitor(AstNode* nodep) {
 	m_modp = NULL;
 	m_stmtCnt = 0;
 	nodep->accept(*this);
@@ -227,7 +219,7 @@ private:
 
 public:
     // CONSTUCTORS
-    InlineCollectVisitor(AstNodeModule* nodep) {  // passed OLD module, not new one
+    explicit InlineCollectVisitor(AstNodeModule* nodep) {  // passed OLD module, not new one
 	nodep->accept(*this);
     }
     virtual ~InlineCollectVisitor() {}
@@ -238,11 +230,14 @@ public:
 
 class InlineRelinkVisitor : public AstNVisitor {
 private:
+    typedef std::set<string> RenamedInterfacesSet;
+
     // NODE STATE
     //  Input:
     //   See InlineVisitor
 
     // STATE
+    RenamedInterfacesSet m_renamedInterfaces;	// Name of renamed interface variables
     AstNodeModule*	m_modp;		// Current module
     AstCell*		m_cellp;	// Cell being cloned
 
@@ -270,6 +265,10 @@ private:
 	nodep->name(name);
 	nodep->iterateChildren(*this);
     }
+    virtual void visit(AstModule* nodep, AstNUser*) {
+	m_renamedInterfaces.clear();
+	nodep->iterateChildren(*this);
+    }
     virtual void visit(AstVar* nodep, AstNUser*) {
 	if (nodep->user2p()) {
 	    // Make an assignment, so we'll trace it properly
@@ -310,23 +309,32 @@ private:
 		nodebp->fileline()->modifyStateInherit(nodep ->fileline());
 	    }
 	}
+	// Iterate won't hit AstIfaceRefDType directly as it is no longer underneath the module
+	if (AstIfaceRefDType* ifacerefp = nodep->dtypep()->castIfaceRefDType()) {
+	    m_renamedInterfaces.insert(nodep->name());
+	    // Each inlined cell that contain an interface variable need to copy the IfaceRefDType and point it to
+	    // the newly cloned interface cell.
+	    AstIfaceRefDType* newdp = ifacerefp->cloneTree(false)->castIfaceRefDType();
+	    nodep->dtypep(newdp);
+	    ifacerefp->addNextHere(newdp);
+	    // Relink to point to newly cloned cell
+	    if (newdp->cellp()) {
+		if (AstCell* newcellp = newdp->cellp()->user4p()->castNode()->castCell()) {
+		    newdp->cellp(newcellp);
+		    newdp->cellName(newcellp->name());
+		    // Tag the old ifacerefp to ensure it leaves no stale reference to the inlined cell.
+		    newdp->user5(false);
+		    ifacerefp->user5(true);
+		}
+	    }
+	}
 	// Variable under the inline cell, need to rename to avoid conflicts
 	// Also clear I/O bits, as it is now local.
 	string name = m_cellp->name() + "__DOT__" + nodep->name();
 	if (!nodep->isFuncLocal()) nodep->inlineAttrReset(name);
 	if (!m_cellp->isTrace()) nodep->trace(false);
 	if (debug()>=9) { nodep->dumpTree(cout,"varchanged:"); }
-	if (debug()>=9) { nodep->valuep()->dumpTree(cout,"varchangei:"); }
-	// Iterate won't hit AstIfaceRefDType directly as it is no longer underneath the module
-	if (AstIfaceRefDType* ifacerefp = nodep->dtypep()->castIfaceRefDType()) {
-	    // Relink to point to newly cloned cell
-	    if (ifacerefp->cellp()) {
-		if (AstCell* newcellp = ifacerefp->cellp()->user4p()->castNode()->castCell()) {
-		    ifacerefp->cellp(newcellp);
-		    ifacerefp->cellName(newcellp->name());
-		}
-	    }
-	}
+	if (debug()>=9 && nodep->valuep()) { nodep->valuep()->dumpTree(cout,"varchangei:"); }
 	nodep->iterateChildren(*this);
     }
     virtual void visit(AstNodeFTask* nodep, AstNUser*) {
@@ -347,7 +355,7 @@ private:
 	    AstVarRef* exprvarrefp = nodep->varp()->user2p()->castNode()->castVarRef();
 	    if (exprconstp) {
 		nodep->replaceWith(exprconstp->cloneTree(true));
-		nodep->deleteTree(); nodep=NULL;
+		nodep->deleteTree(); VL_DANGLING(nodep);
 		return;
 	    }
 	    else if (exprvarrefp) {
@@ -365,7 +373,9 @@ private:
 	string newname = m_cellp->name();
 	if (nodep->inlinedDots() != "") { newname += "." + nodep->inlinedDots(); }
 	nodep->inlinedDots(newname);
-	UINFO(8,"   "<<nodep<<endl);
+	if (m_renamedInterfaces.count(nodep->dotted())) {
+	    nodep->dotted(m_cellp->name() + "__DOT__" + nodep->dotted());
+	}
 	nodep->iterateChildren(*this);
     }
     virtual void visit(AstNodeFTaskRef* nodep, AstNUser*) {
@@ -373,6 +383,9 @@ private:
 	string newname = m_cellp->name();
 	if (nodep->inlinedDots() != "") { newname += "." + nodep->inlinedDots(); }
 	nodep->inlinedDots(newname);
+	if (m_renamedInterfaces.count(nodep->dotted())) {
+	    nodep->dotted(m_cellp->name() + "__DOT__" + nodep->dotted());
+	}
 	UINFO(8,"   "<<nodep<<endl);
 	nodep->iterateChildren(*this);
     }
@@ -422,6 +435,7 @@ class InlineVisitor : public AstNVisitor {
 private:
     // NODE STATE
     // Cleared entire netlist
+    //  AstIfaceRefDType::user5p() // Whether the cell pointed to by this AstIfaceRefDType has been inlined
     //  Input:
     //   AstNodeModule::user1p()	// bool. True to inline this module (from InlineMarkVisitor)
     // Cleared each cell
@@ -430,6 +444,7 @@ private:
     //   AstCell::user4		// AstCell* of the created clone
 
     AstUser4InUse	m_inuser4;
+    AstUser5InUse	m_inuser5;
 
     // STATE
     AstNodeModule*	m_modp;		// Current module
@@ -446,6 +461,14 @@ private:
 	// Iterate modules backwards, in bottom-up order.  Required!
 	nodep->iterateChildrenBackwards(*this);
     }
+    virtual void visit(AstIfaceRefDType* nodep, AstNUser*) {
+	if (nodep->user5()) {
+	    // The cell has been removed so let's make sure we don't leave a reference to it
+	    // This dtype may still be in use by the AstAssignVarScope created earlier
+	    // but that'll get cleared up later
+	    nodep->cellp(NULL);
+	}
+    }
     virtual void visit(AstNodeModule* nodep, AstNUser*) {
 	m_modp = nodep;
 	nodep->iterateChildren(*this);
@@ -486,7 +509,7 @@ private:
 		// need an alias to trace correctly.  If tracing is disabled, we'll
 		// delete it in later optimizations.
 		AstVar* pinOldVarp = pinp->modVarp();
-		AstVar* pinNewVarp = pinOldVarp->clonep()->castVar();
+		AstVar* pinNewVarp = pinOldVarp->clonep();
 
 		AstNode* connectRefp = pinp->exprp();
 		if (!connectRefp->castConst() && !connectRefp->castVarRef()) {
@@ -521,9 +544,9 @@ private:
 	    if (stmtsp) stmtsp->unlinkFrBackWithNext();
 	    if (stmtsp) m_modp->addStmtp(stmtsp);
 	    // Remove the cell
-	    newmodp->deleteTree(); newmodp=NULL; // Clear any leftover ports, etc
+	    newmodp->deleteTree(); VL_DANGLING(newmodp); // Clear any leftover ports, etc
 	    nodep->unlinkFrBack();
-	    pushDeletep(nodep); nodep = NULL;
+	    pushDeletep(nodep); VL_DANGLING(nodep);
 	    if (debug()>=9) { m_modp->dumpTree(cout,"donemod:"); }
 	}
     }
@@ -537,7 +560,7 @@ private:
 
 public:
     // CONSTUCTORS
-    InlineVisitor(AstNode* nodep) {
+    explicit InlineVisitor(AstNode* nodep) {
 	m_modp = NULL;
 	nodep->accept(*this);
     }
@@ -560,7 +583,7 @@ void V3Inline::inlineAll(AstNetlist* nodep) {
     for (AstNodeModule* modp = v3Global.rootp()->modulesp(); modp; modp=nextmodp) {
 	nextmodp = modp->nextp()->castNodeModule();
 	if (modp->user1()) { // Was inlined
-	    modp->unlinkFrBack()->deleteTree(); modp=NULL;
+	    modp->unlinkFrBack()->deleteTree(); VL_DANGLING(modp);
 	}
     }
     V3Global::dumpCheckGlobalTree("inline.tree", 0, v3Global.opt.dumpTreeLevel(__FILE__) >= 3);
