@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -64,8 +64,9 @@ public:
     //
     enum en m_e;
     inline V3LangCode () : m_e(L_ERROR) {}
+    // cppcheck-suppress noExplicitConstructor
     inline V3LangCode (en _e) : m_e(_e) {}
-    V3LangCode (const char* textp);
+    explicit V3LangCode (const char* textp);
     explicit inline V3LangCode (int _e) : m_e(static_cast<en>(_e)) {}
     operator en () const { return m_e; }
 };
