@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -59,8 +59,8 @@ public:
     // CONSTRUCTORS
     LifeState() {}
     ~LifeState() {
-	V3Stats::addStat("Optimizations, Lifetime assign deletions", m_statAssnDel);
-	V3Stats::addStat("Optimizations, Lifetime constant prop", m_statAssnCon);
+	V3Stats::addStatSum("Optimizations, Lifetime assign deletions", m_statAssnDel);
+	V3Stats::addStatSum("Optimizations, Lifetime constant prop", m_statAssnCon);
 	for (vector<AstNode*>::iterator it = m_unlinkps.begin(); it != m_unlinkps.end(); ++it) {
 	    (*it)->unlinkFrBack();
 	    (*it)->deleteTree();
@@ -93,10 +93,10 @@ public:
     LifeVarEntry(SIMPLEASSIGN, AstNodeAssign* assp) {
 	init(true); simpleAssign(assp);
     }
-    LifeVarEntry(COMPLEXASSIGN) {
+    explicit LifeVarEntry(COMPLEXASSIGN) {
 	init(false); complexAssign();
     }
-    LifeVarEntry(CONSUMED) {
+    explicit LifeVarEntry(CONSUMED) {
 	init(false); consumed();
     }
     ~LifeVarEntry() {}
@@ -162,7 +162,7 @@ public:
 		// above our current iteration point.
 		if (debug()>4) oldassp->dumpTree(cout, "       REMOVE/SAMEBLK ");
 		entp->complexAssign();
-		m_statep->pushUnlinkDeletep(oldassp); oldassp=NULL;
+		m_statep->pushUnlinkDeletep(oldassp); VL_DANGLING(oldassp);
 		++m_statep->m_statAssnDel;
 	    }
 	}
@@ -199,7 +199,7 @@ public:
 		    // We'll later constant propagate
 		    UINFO(4,"     replaceconst: "<<varrefp<<endl);
 		    varrefp->replaceWith(constp->cloneTree(false));
-		    varrefp->deleteTree(); varrefp=NULL;
+		    varrefp->deleteTree(); VL_DANGLING(varrefp);
 		    ++m_statep->m_statAssnCon;
 		    return;  // **DONE, no longer a var reference**
 		}
@@ -313,7 +313,7 @@ private:
 	if (nodep->lvalue()) {
 	    m_lifep->complexAssign(vscp);
 	} else {
-	    m_lifep->varUsageReplace(vscp, nodep); nodep=NULL;
+	    m_lifep->varUsageReplace(vscp, nodep); VL_DANGLING(nodep);
 	}
     }
     virtual void visit(AstNodeAssign* nodep, AstNUser*) {
@@ -455,14 +455,14 @@ public:
 	m_sideEffect = false;
 	m_noopt = false;
 	{
-	    // Cpp check bug, it is deleted
-	    // cppcheck-suppress leak
 	    m_lifep = new LifeBlock (NULL, m_statep);
 	    nodep->accept(*this);
-	    delete m_lifep; m_lifep=NULL;
+	    if (m_lifep) { delete m_lifep; m_lifep=NULL; }
 	}
     }
-    virtual ~LifeVisitor() {}
+    virtual ~LifeVisitor() {
+	if (m_lifep) { delete m_lifep; m_lifep=NULL; }
+    }
 };
 
 //######################################################################
