@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -68,7 +68,7 @@ private:
 	    UINFO(9, "  Replace "<<nodep<<" to "<<newvscp<<endl);
 	    AstVarRef* newrefp = new AstVarRef(nodep->fileline(), newvscp, nodep->lvalue());
 	    nodep->replaceWith(newrefp);
-	    nodep->deleteTree(); nodep=NULL;
+	    nodep->deleteTree(); VL_DANGLING(nodep);
 	}
     }
     virtual void visit(AstNodeModule* nodep, AstNUser*) {
@@ -86,7 +86,7 @@ private:
     }
 public:
     // CONSTRUCTORS
-    LifePostElimVisitor(AstTopScope* nodep) {
+    explicit LifePostElimVisitor(AstTopScope* nodep) {
 	nodep->accept(*this);
     }
     virtual ~LifePostElimVisitor() {}
@@ -155,7 +155,7 @@ private:
 		    UINFO(4,"    DELETE "<<nodep<<endl);
 		    // Mark so LifePostElimVisitor will get it
 		    rhsp->varScopep()->user4p(lhsp->varScopep());
-		    nodep->unlinkFrBack()->deleteTree(); nodep=NULL;
+		    nodep->unlinkFrBack()->deleteTree(); VL_DANGLING(nodep);
 		    ++m_statAssnDel;
 		}
 	    }
@@ -178,7 +178,7 @@ private:
     }
 public:
     // CONSTRUCTORS
-    LifePostDlyVisitor(AstNetlist* nodep) {
+    explicit LifePostDlyVisitor(AstNetlist* nodep) {
 	nodep->accept(*this);
     }
     virtual ~LifePostDlyVisitor() {
