@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -113,7 +113,7 @@ void V3LinkLevel::wrapTopCell(AstNetlist* netlistp) {
 
     // Add instance
     AstCell* cellp = new AstCell(newmodp->fileline(),
-				 (v3Global.opt.l2Name() ? "v" : oldmodp->name()),
+				 ((v3Global.opt.l2Name()!="") ? v3Global.opt.l2Name() : oldmodp->name()),
 				 oldmodp->name(),
 				 NULL, NULL, NULL);
     cellp->modp(oldmodp);
