@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -56,10 +56,19 @@ public:
     V3Number& setDouble(double value);
     void setBit (int bit, char value) {		// Note must be pre-zeroed!
 	if (bit>=m_width) return;
-	if (value=='0'||value==0)                       m_value [bit/32] &= ~(1UL<<(bit&31));
-	else {
-	    if (value=='1'||value=='x'||value==1||value==3) m_value [bit/32] |= (1UL<<(bit&31));
-	    if (value=='z'||value=='x'||value==2||value==3) m_valueX[bit/32] |= (1UL<<(bit&31));
+	uint32_t mask = (1UL<<(bit&31));
+	if (value=='0' || value==0) {
+	    m_value [bit/32] &= ~mask;
+	    m_valueX[bit/32] &= ~mask;
+	} else if (value=='1'|| value==1) {
+	    m_value [bit/32] |=  mask;
+	    m_valueX[bit/32] &= ~mask;
+	} else if (value=='z'|| value==2) {
+	    m_value [bit/32] &= ~mask;
+	    m_valueX[bit/32] |=  mask;
+	} else { // X
+	    m_value [bit/32] |=  mask;
+	    m_valueX[bit/32] |=  mask;
 	}
     }
 private:
@@ -120,7 +129,7 @@ private:
 
 public:
     // CONSTRUCTORS
-    V3Number(FileLine* fileline) { init(fileline, 1); }
+    explicit V3Number(FileLine* fileline) { init(fileline, 1); }
     V3Number(FileLine* fileline, int width) { init(fileline, width); }  // 0=unsized
     V3Number(FileLine* fileline, int width, uint32_t value) { init(fileline, width); m_value[0]=value; opCleanThis(); }
     V3Number(FileLine* fileline, const char* source);	// Create from a verilog 32'hxxxx number.
@@ -162,7 +171,7 @@ public:
     // ACCESSORS
     string ascii(bool prefixed=true, bool cleanVerilog=false) const;
     static string quoteNameControls(const string& namein); // Add backslash quotes to strings
-    string displayed(const string& format) const;
+    string displayed(FileLine* fl, const string& format) const;
     static bool displayedFmtLegal(char format);  // Is this a valid format letter?
     int width() const { return m_width; }
     int widthMin() const;	// Minimum width that can represent this number (~== log2(num)+1)
@@ -196,6 +205,7 @@ public:
     double toDouble() const;
     uint32_t toHash() const;
     uint32_t dataWord(int word) const;
+    uint8_t dataByte(int byte) const { return (dataWord(byte/4) >> (8*(byte&3))) & 0xff; }
     uint32_t countOnes() const;
     uint32_t mostSetBitP1() const;	// Highest bit set plus one, IE for 16 return 5, for 0 return 0.
 
