@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -320,7 +320,7 @@ private:
 	    }
 	}
     }
-    virtual void visit(AstConcat* nodep, AstNUser* wp) {
+    virtual void visit(AstConcat* nodep, AstNUser*) {
 	if (m_inAss) {
 	    nodep->lhsp()->iterateAndNext(*this);
 	    int lw = m_childClkWidth;
@@ -329,7 +329,7 @@ private:
 	    m_childClkWidth = lw + rw;  // Pass up
 	}
     }
-    virtual void visit(AstNodeSel* nodep, AstNUser* wp) {
+    virtual void visit(AstNodeSel* nodep, AstNUser*) {
 	if (m_inAss) {
 	    nodep->iterateChildren(*this);
 	    // Pass up result width
@@ -363,7 +363,7 @@ private:
 
 public:
     // CONSTUCTORS
-    OrderClkMarkVisitor(AstNode* nodep) {
+    explicit OrderClkMarkVisitor(AstNode* nodep) {
 	m_hasClk    = false;
 	m_inClocked = false;
 	m_inAss     = false;
@@ -410,7 +410,7 @@ private:
 
 public:
     // CONSTUCTORS
-    OrderClkAssVisitor(AstNode* nodep) {
+    explicit OrderClkAssVisitor(AstNode* nodep) {
 	m_clkAss = false;
 	nodep->accept(*this);
     }
@@ -1265,7 +1265,7 @@ void OrderVisitor::processDomainsIterate(OrderEitherVertex* vertexp) {
 			domainp->dumpTree(cout);
 			cout<<endl;
 		    }
-		    newtreep->deleteTree(); newtreep=NULL;
+		    newtreep->deleteTree(); VL_DANGLING(newtreep);
 		}
 	    }
 	} // next input edgep
@@ -1476,7 +1476,7 @@ void OrderVisitor::processMoveDoneOne(OrderMoveVertex* vertexp) {
 	UINFO(9,"          Clear to "<<(toVertexp->inEmpty()?"[EMP] ":"      ")
 	      <<toVertexp<<endl);
 	// Delete this edge
-	edgep->unlinkDelete(); edgep=NULL;
+	edgep->unlinkDelete(); VL_DANGLING(edgep);
 	if (toVertexp->inEmpty()) {
 	    // If destination node now has all inputs resolved; recurse to move that vertex
 	    // This is thus depth first (before width) which keeps the resulting executable's d-cache happy.
@@ -1530,7 +1530,7 @@ void OrderVisitor::processMoveOne(OrderMoveVertex* vertexp, OrderMoveDomScope* d
 	nodep->unlinkFrBack();
 	if (domainp == m_deleteDomainp) {
 	    UINFO(4," Ordering deleting pre-settled "<<nodep<<endl);
-	    pushDeletep(nodep); nodep=NULL;
+	    pushDeletep(nodep); VL_DANGLING(nodep);
 	} else {
 	    m_pomNewFuncp->addStmtsp(nodep);
 	    if (v3Global.opt.outputSplitCFuncs()) {
@@ -1646,7 +1646,7 @@ void OrderVisitor::process() {
     m_graph.dumpDotFilePrefixed("orderg_done");
     if (0 && debug()) {
 	string dfilename = v3Global.opt.makeDir()+"/"+v3Global.opt.prefix()+"_INT_order.tree";
-	const auto_ptr<ofstream> logp (V3File::new_ofstream(dfilename));
+	const VL_UNIQUE_PTR<ofstream> logp (V3File::new_ofstream(dfilename));
 	if (logp->fail()) v3fatalSrc("Can't write "<<dfilename);
 	m_graph.dump(*logp);
     }
