@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -84,11 +84,11 @@ private:
     typedef deque<pair<AstIfaceRefDType*,AstIfaceRefDType*> > IfaceRefRefs;  // Note may have duplicate entries
 
     // STATE
-    typedef map<AstVar*,AstVar*> VarCloneMap;
+    typedef map<AstNode*,AstNode*> CloneMap;
     struct ModInfo {
 	AstNodeModule*	m_modp;		// Module with specified name
-	VarCloneMap	m_cloneMap;	// Map of old-varp -> new cloned varp
-	ModInfo(AstNodeModule* modp) { m_modp=modp; }
+	CloneMap	m_cloneMap;	// Map of old-varp -> new cloned varp
+	explicit ModInfo(AstNodeModule* modp) { m_modp=modp; }
     };
     typedef map<string,ModInfo> ModNameMap;
     ModNameMap	m_modNameMap;	// Hash of created module flavors by name
@@ -108,6 +108,8 @@ private:
     typedef deque<AstCell*> CellList;
     CellList	m_cellps;	// Cells left to process (in this module)
 
+    string m_unlinkedTxt;	// Text for AstUnlinkedRef
+
     // METHODS
     static int debug() {
 	static int level = -1;
@@ -126,10 +128,14 @@ private:
 		    varp->user4(usedLetter[static_cast<int>(ch)]*256 + ch);
 		    usedLetter[static_cast<int>(ch)]++;
 		}
+	    } else if (AstParamTypeDType* typep = stmtp->castParamTypeDType()) {
+	        char ch = 'T';
+	        typep->user4(usedLetter[static_cast<int>(ch)]*256 + ch);
+	        usedLetter[static_cast<int>(ch)]++;
 	    }
 	}
     }
-    string paramSmallName(AstNodeModule* modp, AstVar* varp) {
+    string paramSmallName(AstNodeModule* modp, AstNode* varp) {
 	if (varp->user4()<=1) {
 	    makeSmallNames(modp);
 	}
@@ -137,16 +143,16 @@ private:
 	char ch   = varp->user4()&255;
 	string st = cvtToStr(ch);
 	while (index) {
-	    st += cvtToStr(char((index%26)+'A'));
+	    st += cvtToStr(char((index%25)+'A'));
 	    index /= 26;
 	}
 	return st;
     }
     string paramValueNumber(AstNode* nodep) {
-	// Given a compilcated object create a number to use for param module assignment
+	// Given a complicated object create a number to use for param module assignment
 	// Ideally would be relatively stable if design changes (not use pointer value),
 	// and must return same value given same input node
-	// Return must presently be numberic so doesn't collide with 'small' alphanumeric parameter names
+	// Return must presently be numeric so doesn't collide with 'small' alphanumeric parameter names
 	ValueMap::iterator it = m_valueMap.find(nodep);
 	if (it != m_valueMap.end()) {
 	    return cvtToStr(it->second);
@@ -160,17 +166,46 @@ private:
 	    else { m_nextValueMap.insert(make_pair(bucket, offset + 1)); }
 	    int num = bucket + offset * BUCKETS;
 	    m_valueMap.insert(make_pair(nodep, num));
-	    return cvtToStr(num);
+	    // 'z' just to make sure we don't collide with a normal non-hashed number
+	    return (string)"z"+cvtToStr(num);
 	}
     }
-    void relinkPins(VarCloneMap* clonemapp, AstPin* startpinp) {
+    void collectPins(CloneMap* clonemapp, AstNodeModule* modp) {
+	// Grab all I/O so we can remap our pins later
+	for (AstNode* stmtp=modp->stmtsp(); stmtp; stmtp = stmtp->nextp()) {
+	    if (AstVar* varp = stmtp->castVar()) {
+		if (varp->isIO() || varp->isGParam() || varp->isIfaceRef()) {
+		    // Cloning saved a pointer to the new node for us, so just follow that link.
+		    AstVar* oldvarp = varp->clonep();
+		    //UINFO(8,"Clone list 0x"<<hex<<(uint32_t)oldvarp<<" -> 0x"<<(uint32_t)varp<<endl);
+		    clonemapp->insert(make_pair(oldvarp, varp));
+		}
+	    }
+	    else if (AstParamTypeDType* ptp = stmtp->castParamTypeDType()) {
+		if (ptp->isGParam()) {
+		    AstParamTypeDType* oldptp = ptp->clonep();
+		    clonemapp->insert(make_pair(oldptp, ptp));
+		}
+	    }
+	}
+    }
+    void relinkPins(CloneMap* clonemapp, AstPin* startpinp) {
 	for (AstPin* pinp = startpinp; pinp; pinp=pinp->nextp()->castPin()) {
-	    if (!pinp->modVarp()) pinp->v3fatalSrc("Not linked?\n");
-	    // Find it in the clone structure
-	    //UINFO(8,"Clone find 0x"<<hex<<(uint32_t)pinp->modVarp()<<endl);
-	    VarCloneMap::iterator cloneiter = clonemapp->find(pinp->modVarp());
-	    UASSERT(cloneiter != clonemapp->end(), "Couldn't find pin in clone list");
-	    pinp->modVarp(cloneiter->second);
+	    if (pinp->modVarp()) {
+		// Find it in the clone structure
+		//UINFO(8,"Clone find 0x"<<hex<<(uint32_t)pinp->modVarp()<<endl);
+		CloneMap::iterator cloneiter = clonemapp->find(pinp->modVarp());
+		UASSERT(cloneiter != clonemapp->end(), "Couldn't find pin in clone list");
+		pinp->modVarp(cloneiter->second->castVar());
+	    }
+	    else if (pinp->modPTypep()) {
+		CloneMap::iterator cloneiter = clonemapp->find(pinp->modPTypep());
+		UASSERT(cloneiter != clonemapp->end(), "Couldn't find pin in clone list");
+		pinp->modPTypep(cloneiter->second->castParamTypeDType());
+	    }
+	    else {
+		pinp->v3fatalSrc("Not linked?\n");
+	    }
 	}
     }
     void visitCell(AstCell* nodep);
@@ -248,6 +283,47 @@ private:
     virtual void visit(AstVarRef* nodep, AstNUser*) {
 	if (nodep->varp()) nodep->varp()->iterate(*this);
     }
+    virtual void visit(AstVarXRef* nodep, AstNUser*) {
+	nodep->varp(NULL);  // Needs relink, as may remove pointed-to var
+    }
+
+    virtual void visit(AstUnlinkedRef* nodep, AstNUser*) {
+	AstVarXRef* varxrefp = nodep->op1p()->castVarXRef();
+	AstNodeFTaskRef* taskrefp = nodep->op1p()->castNodeFTaskRef();
+	if (varxrefp) {
+	    m_unlinkedTxt = varxrefp->dotted();
+	} else if (taskrefp) {
+	    m_unlinkedTxt = taskrefp->dotted();
+	} else {
+	    nodep->v3fatalSrc("Unexpected AstUnlinkedRef node");
+	    return;
+	}
+	nodep->cellrefp()->iterate(*this);
+
+	if (varxrefp) {
+	    varxrefp->dotted(m_unlinkedTxt);
+	} else {
+	    taskrefp->dotted(m_unlinkedTxt);
+	}
+	nodep->replaceWith(nodep->op1p()->unlinkFrBack());
+	pushDeletep(nodep); VL_DANGLING(nodep);
+    }
+    virtual void visit(AstCellArrayRef* nodep, AstNUser*) {
+	V3Const::constifyParamsEdit(nodep->selp());
+	if (AstConst* constp = nodep->selp()->castConst()) {
+	    string index = AstNode::encodeNumber(constp->toSInt());
+	    string replacestr = nodep->name() + "__BRA__??__KET__";
+	    size_t pos = m_unlinkedTxt.find(replacestr);
+	    if (pos == string::npos) {
+		nodep->v3error("Could not find array index in unlinked text: '" << m_unlinkedTxt << "' for node: " << nodep);
+		return;
+	    }
+	    m_unlinkedTxt.replace(pos, replacestr.length(), nodep->name() + "__BRA__"+index+"__KET__");
+	} else {
+	    nodep->v3error("Could not expand constant selection inside dotted reference: "<<nodep->selp()->prettyName());
+	    return;
+	}
+    }
 
     // Generate Statements
     virtual void visit(AstGenerate* nodep, AstNUser*) {
@@ -262,7 +338,7 @@ private:
 	} else {
 	    nodep->unlinkFrBack();
 	}
-	nodep->deleteTree(); nodep=NULL;
+	nodep->deleteTree(); VL_DANGLING(nodep);
     }
     virtual void visit(AstGenIf* nodep, AstNUser*) {
 	UINFO(9,"  GENIF "<<nodep<<endl);
@@ -283,7 +359,7 @@ private:
 	    } else {
 		nodep->unlinkFrBack();
 	    }
-	    nodep->deleteTree(); nodep=NULL;
+	    nodep->deleteTree(); VL_DANGLING(nodep);
 	    // Normal edit rules will now recurse the replacement
 	} else {
 	    nodep->condp()->v3error("Generate If condition must evaluate to constant");
@@ -310,7 +386,7 @@ private:
 	    string beginName = nodep->name();
 	    // Leave the original Begin, as need a container for the (possible) GENVAR
 	    // Note V3Unroll will replace some AstVarRef's to the loop variable with constants
-	    V3Unroll::unrollGen(forp, beginName); forp=NULL;
+	    V3Unroll::unrollGen(forp, beginName); VL_DANGLING(forp);
 	    // Blocks were constructed under the special begin, move them up
 	    // Note forp is null, so grab statements again
 	    if (AstNode* stmtsp = nodep->genforp()) {
@@ -339,8 +415,7 @@ private:
 	    for (AstNode* ep = itemp->condsp(); ep; ) {
 		AstNode* nextp = ep->nextp(); //May edit list
 		ep->iterateAndNext(*this);
-		V3Const::constifyParamsEdit(ep); ep=NULL; // ep may change
-		// cppcheck-suppress redundantAssignment
+		V3Const::constifyParamsEdit(ep); VL_DANGLING(ep); // ep may change
 		ep = nextp;
 	    }
 	}
@@ -372,7 +447,7 @@ private:
 	    nodep->replaceWith(keepp);
 	}
 	else nodep->unlinkFrBack();
-	nodep->deleteTree(); nodep=NULL;
+	nodep->deleteTree(); VL_DANGLING(nodep);
     }
 
     // Default: Just iterate
@@ -382,7 +457,7 @@ private:
 
 public:
     // CONSTUCTORS
-    ParamVisitor(AstNetlist* nodep) {
+    explicit ParamVisitor(AstNetlist* nodep) {
 	m_longId = 0;
 	//
 	nodep->accept(*this);
@@ -415,26 +490,50 @@ void ParamVisitor::visitCell(AstCell* nodep) {
 	if (debug()>8) nodep->paramsp()->dumpTreeAndNext(cout,"-cellparams:\t");
 	for (AstPin* pinp = nodep->paramsp(); pinp; pinp=pinp->nextp()->castPin()) {
 	    if (!pinp->exprp()) continue; // No-connect
-	    AstVar* modvarp = pinp->modVarp();
-	    if (!modvarp) {
-		pinp->v3error("Parameter not found in sub-module: Param "<<pinp->name()<<" of "<<nodep->prettyName());
-	    } else if (!modvarp->isGParam()) {
-		pinp->v3error("Attempted parameter setting of non-parameter: Param "<<pinp->name()<<" of "<<nodep->prettyName());
-	    } else {
-		AstConst* constp = pinp->exprp()->castConst();
-		AstConst* origconstp = modvarp->valuep()->castConst();
-		if (!constp) {
-		    //if (debug()) pinp->dumpTree(cout,"error:");
-		    pinp->v3error("Can't convert defparam value to constant: Param "<<pinp->name()<<" of "<<nodep->prettyName());
-		    pinp->exprp()->replaceWith(new AstConst(pinp->fileline(), V3Number(pinp->fileline(), modvarp->width(), 0)));
-		} else if (origconstp && constp->sameTree(origconstp)) {
-		    // Setting parameter to its default value.  Just ignore it.
-		    // This prevents making additional modules, and makes coverage more
-		    // obvious as it won't show up under a unique module page name.
+	    if (AstVar* modvarp = pinp->modVarp()) {
+		if (!modvarp->isGParam()) {
+		    pinp->v3error("Attempted parameter setting of non-parameter: Param "<<pinp->prettyName()<<" of "<<nodep->prettyName());
 		} else {
-		    longname += "_" + paramSmallName(nodep->modp(),pinp->modVarp())+constp->num().ascii(false);
-		    any_overrides = true;
+		    AstConst* exprp = pinp->exprp()->castConst();
+		    AstConst* origp = modvarp->valuep()->castConst();
+		    if (!exprp) {
+			//if (debug()) pinp->dumpTree(cout,"error:");
+			pinp->v3error("Can't convert defparam value to constant: Param "<<pinp->name()<<" of "<<nodep->prettyName());
+			pinp->exprp()->replaceWith(new AstConst(pinp->fileline(), V3Number(pinp->fileline(), modvarp->width(), 0)));
+		    } else if (origp && exprp->sameTree(origp)) {
+			// Setting parameter to its default value.  Just ignore it.
+			// This prevents making additional modules, and makes coverage more
+			// obvious as it won't show up under a unique module page name.
+		    } else if (exprp->num().isDouble()
+			       || exprp->num().isString()
+			       ||  exprp->num().isFourState()) {
+			longname += "_" + paramSmallName(nodep->modp(),modvarp)+paramValueNumber(exprp);
+			any_overrides = true;
+		    } else {
+			longname += "_" + paramSmallName(nodep->modp(),modvarp)+exprp->num().ascii(false);
+			any_overrides = true;
+		    }
 		}
+	    } else if (AstParamTypeDType* modvarp = pinp->modPTypep()) {
+		AstNodeDType* exprp = pinp->exprp()->castNodeDType();
+		AstNodeDType* origp = modvarp->subDTypep();
+		if (!exprp) {
+		    pinp->v3error("Parameter type pin value isn't a type: Param "<<pinp->prettyName()<<" of "<<nodep->prettyName());
+		} else if (!origp) {
+		    pinp->v3error("Parameter type variable isn't a type: Param "<<modvarp->prettyName());
+		} else {
+		    UINFO(9,"Parameter type assignment expr="<<exprp<<" to "<<origp<<endl);
+		    if (origp && exprp->sameTree(origp)) {
+			// Setting parameter to its default value.  Just ignore it.
+			// This prevents making additional modules, and makes coverage more
+			// obvious as it won't show up under a unique module page name.
+		    } else {
+			longname += "_" + paramSmallName(nodep->modp(),modvarp)+paramValueNumber(exprp);
+			any_overrides = true;
+		    }
+		}
+	    } else {
+		pinp->v3error("Parameter not found in sub-module: Param "<<pinp->prettyName()<<" of "<<nodep->prettyName());
 	    }
 	}
 	IfaceRefRefs ifaceRefRefs;
@@ -442,16 +541,44 @@ void ParamVisitor::visitCell(AstCell* nodep) {
 	    AstVar* modvarp = pinp->modVarp();
 	    if (modvarp->isIfaceRef()) {
 		AstIfaceRefDType* portIrefp = modvarp->subDTypep()->castIfaceRefDType();
-		//UINFO(9,"     portIfaceRef "<<portIrefp<<endl);
-		if (!pinp->exprp()
-		    || !pinp->exprp()->castVarRef()
-		    || !pinp->exprp()->castVarRef()->varp()
-		    || !pinp->exprp()->castVarRef()->varp()->subDTypep()
-		    || !pinp->exprp()->castVarRef()->varp()->subDTypep()->castIfaceRefDType()) {
+		if (!portIrefp) {
+		    portIrefp = modvarp->subDTypep()->castUnpackArrayDType()->subDTypep()->castIfaceRefDType();
+		}
+
+		AstIfaceRefDType* pinIrefp = NULL;
+		AstNode* exprp = pinp->exprp();
+		if (exprp
+		    && exprp->castVarRef()
+		    && exprp->castVarRef()->varp()
+		    && exprp->castVarRef()->varp()->subDTypep()
+		    && exprp->castVarRef()->varp()->subDTypep()->castIfaceRefDType())
+		    pinIrefp = exprp->castVarRef()->varp()->subDTypep()->castIfaceRefDType();
+		else if (exprp
+			 && exprp->op1p()
+			 && exprp->op1p()->castVarRef()
+			 && exprp->op1p()->castVarRef()->varp()
+			 && exprp->op1p()->castVarRef()->varp()->subDTypep()
+			 && exprp->op1p()->castVarRef()->varp()->subDTypep()->castUnpackArrayDType()
+			 && exprp->op1p()->castVarRef()->varp()->subDTypep()->castUnpackArrayDType()->subDTypep()
+			 && exprp->op1p()->castVarRef()->varp()->subDTypep()->castUnpackArrayDType()->subDTypep()->castIfaceRefDType())
+		    pinIrefp = exprp->op1p()->castVarRef()->varp()->subDTypep()->castUnpackArrayDType()->subDTypep()->castIfaceRefDType();
+		else if (exprp
+			 && exprp->castVarRef()
+			 && exprp->castVarRef()->varp()
+			 && exprp->castVarRef()->varp()->subDTypep()
+			 && exprp->castVarRef()->varp()->subDTypep()->castUnpackArrayDType()
+			 && exprp->castVarRef()->varp()->subDTypep()->castUnpackArrayDType()->subDTypep()
+			 && exprp->castVarRef()->varp()->subDTypep()->castUnpackArrayDType()->subDTypep()->castIfaceRefDType())
+		    pinIrefp = exprp->castVarRef()->varp()->subDTypep()->castUnpackArrayDType()->subDTypep()->castIfaceRefDType();
+
+		UINFO(9,"     portIfaceRef "<<portIrefp<<endl);
+
+		if (!portIrefp) {
+		    pinp->v3error("Interface port '"<<modvarp->prettyName()<<"' is not an interface" << modvarp);
+		} else if (!pinIrefp) {
 		    pinp->v3error("Interface port '"<<modvarp->prettyName()<<"' is not connected to interface/modport pin expression");
 		} else {
-		    AstIfaceRefDType* pinIrefp = pinp->exprp()->castVarRef()->varp()->subDTypep()->castIfaceRefDType();
-		    //UINFO(9,"     pinIfaceRef "<<pinIrefp<<endl);
+		    UINFO(9,"     pinIfaceRef "<<pinIrefp<<endl);
 		    if (portIrefp->ifaceViaCellp() != pinIrefp->ifaceViaCellp()) {
 			UINFO(9,"     IfaceRefDType needs reconnect  "<<pinIrefp<<endl);
 			longname += "_" + paramSmallName(nodep->modp(),pinp->modVarp())+paramValueNumber(pinIrefp);
@@ -467,6 +594,7 @@ void ParamVisitor::visitCell(AstCell* nodep) {
 	} else {
 	    // If the name is very long, we don't want to overwhelm the filename limit
 	    // We don't do this always, as it aids debugability to have intuitive naming.
+	    // TODO can use new V3Name hash replacement instead of this
 	    string newname = longname;
 	    if (longname.length()>30) {
 		LongMap::iterator iter = m_longMap.find(longname);
@@ -496,22 +624,12 @@ void ParamVisitor::visitCell(AstCell* nodep) {
 
 		m_modNameMap.insert(make_pair(modp->name(), ModInfo(modp)));
 		iter = m_modNameMap.find(newname);
-		VarCloneMap* clonemapp = &(iter->second.m_cloneMap);
+		CloneMap* clonemapp = &(iter->second.m_cloneMap);
 		UINFO(4,"     De-parameterize to new: "<<modp<<endl);
 
 		// Grab all I/O so we can remap our pins later
 		// Note we allow multiple users of a parameterized model, thus we need to stash this info.
-		for (AstNode* stmtp=modp->stmtsp(); stmtp; stmtp = stmtp->nextp()) {
-		    if (AstVar* varp = stmtp->castVar()) {
-			if (varp->isIO() || varp->isGParam() || varp->isIfaceRef()) {
-			    // Cloning saved a pointer to the new node for us, so just follow that link.
-			    AstVar* oldvarp = varp->clonep()->castVar();
-			    //UINFO(8,"Clone list 0x"<<hex<<(uint32_t)oldvarp<<" -> 0x"<<(uint32_t)varp<<endl);
-			    clonemapp->insert(make_pair(oldvarp, varp));
-			}
-		    }
-		}
-
+		collectPins(clonemapp, modp);
 		// Relink parameter vars to the new module
 		relinkPins(clonemapp, nodep->paramsp());
 
@@ -519,7 +637,7 @@ void ParamVisitor::visitCell(AstCell* nodep) {
 		for (IfaceRefRefs::iterator it=ifaceRefRefs.begin(); it!=ifaceRefRefs.end(); ++it) {
 		    AstIfaceRefDType* portIrefp = it->first;
 		    AstIfaceRefDType* pinIrefp = it->second;
-		    AstIfaceRefDType* cloneIrefp = portIrefp->clonep()->castIfaceRefDType();
+		    AstIfaceRefDType* cloneIrefp = portIrefp->clonep();
 		    UINFO(8,"     IfaceOld "<<portIrefp<<endl);
 		    UINFO(8,"     IfaceTo  "<<pinIrefp<<endl);
 		    if (!cloneIrefp) portIrefp->v3fatalSrc("parameter clone didn't hit AstIfaceRefDType");
@@ -531,15 +649,26 @@ void ParamVisitor::visitCell(AstCell* nodep) {
 		// Assign parameters to the constants specified
 		// DOES clone() so must be finished with module clonep() before here
 		for (AstPin* pinp = nodep->paramsp(); pinp; pinp=pinp->nextp()->castPin()) {
-		    AstVar* modvarp = pinp->modVarp();
-		    if (modvarp && pinp->exprp()) {
-			AstConst* constp = pinp->exprp()->castConst();
-			// Remove any existing parameter
-			if (modvarp->valuep()) modvarp->valuep()->unlinkFrBack()->deleteTree();
-			// Set this parameter to value requested by cell
-			modvarp->valuep(constp->cloneTree(false));
+		    if (pinp->exprp()) {
+			if (AstVar* modvarp = pinp->modVarp()) {
+			    AstConst* constp = pinp->exprp()->castConst();
+			    // Remove any existing parameter
+			    if (modvarp->valuep()) modvarp->valuep()->unlinkFrBack()->deleteTree();
+			    // Set this parameter to value requested by cell
+			    modvarp->valuep(constp->cloneTree(false));
+			}
+			else if (AstParamTypeDType* modptp = pinp->modPTypep()) {
+			    AstNodeDType* dtypep = pinp->exprp()->castNodeDType();
+			    if (!dtypep) pinp->v3fatalSrc("unlinked param dtype");
+			    if (modptp->childDTypep()) pushDeletep(modptp->childDTypep()->unlinkFrBack());
+			    // Set this parameter to value requested by cell
+			    modptp->childDTypep(dtypep->cloneTree(false));
+			    // Later V3LinkDot will convert the ParamDType to a Typedef
+			    // Not done here as may be localparams, etc, that also need conversion
+			}
 		    }
 		}
+
 	    } else {
 		UINFO(4,"     De-parameterize to old: "<<modp<<endl);
 	    }
@@ -549,7 +678,7 @@ void ParamVisitor::visitCell(AstCell* nodep) {
 	    nodep->modName(newname);
 
 	    // We need to relink the pins to the new module
-	    VarCloneMap* clonemapp = &(iter->second.m_cloneMap);
+	    CloneMap* clonemapp = &(iter->second.m_cloneMap);
 	    relinkPins(clonemapp, nodep->pinsp());
 	    UINFO(8,"     Done with "<<modp<<endl);
 	} // if any_overrides
