@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2000-2015 by Wilson Snyder.  This program is free software;
+// Copyright 2000-2016 by Wilson Snyder.  This program is free software;
 // you can redistribute it and/or modify it under the terms of either the
 // GNU Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -1074,7 +1074,7 @@ int V3PreProcImp::getStateToken() {
 		    out = defineSubst(refp);
 		    //NOP: out = m_preprocp->defSubstitute(out);
 		}
-		m_defRefs.pop();  refp=NULL;
+		m_defRefs.pop(); VL_DANGLING(refp);
 		if (m_defRefs.empty()) {
 		    statePop();
 		    if (!m_off) unputDefrefString(out);
