@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -294,7 +294,7 @@ private:
     }
 public:
     // CONSTUCTORS
-    ScopeVisitor(AstNetlist* nodep) {
+    explicit ScopeVisitor(AstNetlist* nodep) {
 	m_aboveCellp = NULL;
 	m_aboveScopep = NULL;
 	m_modp = NULL;
@@ -334,7 +334,9 @@ private:
 	    nodep->iterateChildren(*this);
 	} else {
 	    // A block that was just moved under a scope, Kill it.
-	    nodep->unlinkFrBack()->deleteTree(); nodep=NULL;
+	    // Certain nodes can be referenced later in this pass, notably
+	    // an FTaskRef needs to access the FTask to find the cloned task
+	    pushDeletep(nodep->unlinkFrBack()); VL_DANGLING(nodep);
 	}
     }
 
@@ -402,7 +404,7 @@ private:
     }
 public:
     // CONSTUCTORS
-    ScopeCleanupVisitor(AstNetlist* nodep) {
+    explicit ScopeCleanupVisitor(AstNetlist* nodep) {
 	m_scopep = NULL;
 	nodep->accept(*this);
     }
