@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -255,12 +255,16 @@ private:
     virtual void visit(AstVarRef* nodep, AstNUser*) {
 	if (jumpingOver(nodep)) return;
 	if (!optimizable()) return;  // Accelerate
+	nodep->varp()->iterateChildren(*this);
 	AstNode* vscp = varOrScope(nodep);
 
 	// We can't have non-delayed assignments with same value on LHS and RHS
 	// as we don't figure out variable ordering.
 	// Delayed is OK though, as we'll decode the next state separately.
-	if (!nodep->varp()->dtypeSkipRefp()->castBasicDType()) clearOptimizable(nodep,"Array references/not basic");
+	if (!nodep->varp()->dtypeSkipRefp()->castBasicDType()
+            && !nodep->varp()->dtypeSkipRefp()->castPackArrayDType()
+            && !nodep->varp()->dtypeSkipRefp()->castStructDType())
+            clearOptimizable(nodep,"Array references/not basic");
 	if (nodep->lvalue()) {
 	    if (m_inDlyAssign) {
 		if (!(vscp->user1() & VU_LVDLY)) {
@@ -279,10 +283,10 @@ private:
 		if (!m_params && (vscp->user1() & VU_LV)) clearOptimizable(nodep,"Var write & read");
 		vscp->user1( vscp->user1() | VU_RV);
 		bool isConst = nodep->varp()->isParam();
-		AstConst* constp = (isConst ? nodep->varp()->valuep()->castConst() : NULL);
-		if (isConst && constp) { // Propagate PARAM constants for constant function analysis
+		V3Number* nump = isConst ? fetchNumberNull(nodep->varp()->valuep()) : NULL;
+		if (isConst && nump) { // Propagate PARAM constants for constant function analysis
 		    if (!m_checkOnly && optimizable()) {
-			newNumber(vscp)->opAssign(constp->num());
+			newNumber(vscp)->opAssign(*nump);
 		    }
 		} else {
 		    if (m_checkOnly) varRefCb (nodep);
@@ -342,6 +346,19 @@ private:
 	    setNumber(nodep, &(nodep->num()));
 	}
     }
+    virtual void visit(AstEnumItemRef* nodep, AstNUser*) {
+	checkNodeInfo(nodep);
+	if (!nodep->itemp()) nodep->v3fatalSrc("Not linked");
+	if (!m_checkOnly && optimizable()) {
+            AstNode* valuep = nodep->itemp()->valuep();
+	    if (valuep) {
+	        valuep->iterateAndNext(*this);
+	        newNumber(nodep)->opAssign(*fetchNumber(valuep));
+            } else {
+                clearOptimizable(nodep, "No value found for enum item");
+            }
+        }
+    }
     virtual void visit(AstNodeUniop* nodep, AstNUser*) {
 	if (!optimizable()) return;  // Accelerate
 	checkNodeInfo(nodep);
@@ -435,6 +452,55 @@ private:
 	    }
 	}
     }
+
+    bool handleAssignSel(AstNodeAssign* nodep, AstSel* selp, AstVarRef** outVarrefp, int depth) {
+	checkNodeInfo(selp);
+	AstVarRef* varrefp = selp->fromp()->castVarRef();
+	if (!varrefp) {
+	    selp = selp->lhsp()->castSel();
+	    if (selp) {
+		if (!handleAssignSel(nodep, selp, &varrefp, depth+1)) {
+		    clearOptimizable(nodep, "Select LHS isn't simple variable");
+		    return false;
+		}
+	    }
+	}
+
+	if (m_checkOnly) {
+	    nodep->iterateChildren(*this);
+	} else {
+	    selp->lsbp()->iterateAndNext(*this);
+	    nodep->rhsp()->iterateAndNext(*this);
+
+	    if (optimizable()) {
+		if (varrefp) {
+		    AstNode* vscp = varOrScope(varrefp);
+		    V3Number outnum = V3Number(nodep->fileline());
+		    if (V3Number* vscpnump = fetchOutNumberNull(vscp)) {
+			outnum = *vscpnump;
+		    } else if (V3Number* vscpnump = fetchNumberNull(vscp)) {
+			outnum = *vscpnump;
+		    } else {  // Assignment to unassigned variable, all bits are X or 0
+			outnum = V3Number(nodep->fileline(), varrefp->varp()->widthMin());
+			if (varrefp->varp()->basicp() && varrefp->varp()->basicp()->isZeroInit()) {
+			    outnum.setAllBits0();
+			} else {
+			    outnum.setAllBitsX();
+			}
+		    }
+		    if (depth == 0) {
+			outnum.opSelInto(*fetchNumber(nodep->rhsp()),
+					 *fetchNumber(selp->lsbp()),
+					 selp->widthConst());
+			assignOutNumber(nodep, vscp, &outnum);
+		    }
+		}
+	    }
+	}
+	if (outVarrefp) *outVarrefp = varrefp;
+	return true;
+    }
+
     virtual void visit(AstNodeAssign* nodep, AstNUser*) {
 	if (jumpingOver(nodep)) return;
 	if (!optimizable()) return;  // Accelerate
@@ -446,41 +512,10 @@ private:
 	    if (m_anyAssignDly) clearOptimizable(nodep, "Mix of dly/non-dly assigns");
 	    m_anyAssignComb = true;
 	}
+
 	if (AstSel* selp = nodep->lhsp()->castSel()) {
 	    if (!m_params) { clearOptimizable(nodep, "LHS has select"); return; }
-	    checkNodeInfo(selp);
-	    AstVarRef* varrefp = selp->fromp()->castVarRef();
-	    if (!varrefp) { 
-		clearOptimizable(nodep, "Select LHS isn't simple variable");
-		return;
-	    }
-	    if (m_checkOnly) {
-		nodep->iterateChildren(*this);
-	    } else {
-		selp->lsbp()->iterateAndNext(*this);
-		nodep->rhsp()->iterateAndNext(*this);
-		if (optimizable()) {
-		    AstNode* vscp = varOrScope(varrefp);
-		    if (optimizable()) {
-			V3Number outnum (nodep->fileline(), varrefp->varp()->widthMin());
-			if (V3Number* outnump = fetchOutNumberNull(vscp)) {
-			    outnum = *outnump;
-			} else if (V3Number* outnump = fetchNumberNull(vscp)) {
-			    outnum = *outnump;
-			} else {  // Assignment to unassigned variable, all bits are X or 0
-			    if (varrefp->varp()->basicp() && varrefp->varp()->basicp()->isZeroInit()) {
-				outnum.setAllBits0();
-			    } else {
-				outnum.setAllBitsX();
-			    }
-			}
-			outnum.opSelInto(*fetchNumber(nodep->rhsp()),
-					 *fetchNumber(selp->lsbp()),
-					 selp->widthConst());
-			assignOutNumber(nodep, vscp, &outnum);
-		    }
-		}
-	    }
+	    handleAssignSel(nodep, selp, NULL, 0);
 	}
 	else if (!nodep->lhsp()->castVarRef()) {
 	    clearOptimizable(nodep, "LHS isn't simple variable");
@@ -564,6 +599,7 @@ private:
 	}
     }
     virtual void visit(AstStop* nodep, AstNUser*) {
+	if (jumpingOver(nodep)) return;
 	if (m_params) {  // This message seems better than an obscure $stop
 	    // The spec says $stop is just ignored, it seems evil to ignore assertions
 	    clearOptimizable(nodep,"$stop executed during function constification; maybe indicates assertion firing");
@@ -637,8 +673,7 @@ private:
 	UINFO(5,"   FUNCREF "<<nodep<<endl);
 	if (!m_params) { badNodeType(nodep); return; }
 	AstNodeFTask* funcp = nodep->taskp()->castNodeFTask(); if (!funcp) nodep->v3fatalSrc("Not linked");
-	// cppcheck-suppress redundantAssignment
-	if (m_params) { V3Width::widthParamsEdit(funcp); } funcp=NULL; // Make sure we've sized the function
+	if (m_params) { V3Width::widthParamsEdit(funcp); } VL_DANGLING(funcp); // Make sure we've sized the function
 	funcp = nodep->taskp()->castNodeFTask(); if (!funcp) nodep->v3fatalSrc("Not linked");
 	// Apply function call values to function
 	V3TaskConnects tconnects = V3Task::taskConnects(nodep, nodep->taskp()->stmtsp());
@@ -680,9 +715,78 @@ private:
 	if (!m_params) { badNodeType(nodep); return; }
     }
 
+    virtual void visit(AstSFormatF *nodep, AstNUser*) {
+	if (jumpingOver(nodep)) return;
+	nodep->iterateChildren(*this);
+	if (m_params) {
+	    AstNode* nextArgp = nodep->exprsp();
+
+	    string result = "";
+	    string format = nodep->text();
+	    string::const_iterator pos = format.begin();
+	    bool inPct = false;
+	    for (; pos != format.end(); ++pos) {
+		if (!inPct && pos[0] == '%') {
+		    inPct = true;
+		} else if (!inPct) {   // Normal text
+		    result += *pos;
+		} else { // Format character
+		    AstNode* argp = nextArgp;
+		    inPct = false;
+		    nextArgp = nextArgp->nextp();
+
+		    if (V3Number::displayedFmtLegal(tolower(pos[0]))) {
+			V3Number* nump = fetchNumberNull(argp);
+			if (!nump) {
+			    clearOptimizable(nodep, "Argument for $display like statement is not constant");
+			    break;
+			}
+			string format = string("%") + pos[0];
+			result += nump->displayed(nodep->fileline(), format);
+		    } else {
+			switch (tolower(pos[0])) {
+			case '%':
+			    result += "%";
+			    break;
+			default:
+			    clearOptimizable(nodep, "Unknown $display-like format code.");
+			    break;
+			}
+		    }
+		}
+	    }
+	    nodep->text(result);
+	}
+    }
+
+    virtual void visit(AstDisplay *nodep, AstNUser*) {
+	if (jumpingOver(nodep)) return;
+	nodep->iterateChildren(*this);
+	if (m_params) {
+	    switch (nodep->displayType()) {
+	    case AstDisplayType::DT_DISPLAY:  // FALLTHRU
+	    case AstDisplayType::DT_INFO:
+		v3warn(USERINFO, nodep->fmtp()->text());
+		break;
+	    case AstDisplayType::DT_ERROR:
+		v3warn(USERERROR, nodep->fmtp()->text());
+		break;
+	    case AstDisplayType::DT_WARNING:
+		v3warn(USERWARN, nodep->fmtp()->text());
+		break;
+	    case AstDisplayType::DT_FATAL:
+		v3warn(USERFATAL, nodep->fmtp()->text());
+		break;
+	    case AstDisplayType::DT_WRITE:  // FALLTHRU
+	    default:
+		clearOptimizable(nodep, "Unexpected display type");
+	    }
+	}
+    }
+
     // default
     // These types are definately not reducable
-    //   AstCoverInc, AstDisplay, AstArraySel, AstStop, AstFinish,
+    //   AstCoverInc, AstArraySel, AstFinish,
     //   AstRand, AstTime, AstUCFunc, AstCCall, AstCStmt, AstUCStmt
     virtual void visit(AstNode* nodep, AstNUser*) {
 	if (jumpingOver(nodep)) return;
@@ -726,7 +830,7 @@ public:
 	// Move all allocated numbers to the free pool
 	m_numFreeps = m_numAllps;
     }
-    void mainTableCheck   (AstNode* nodep) {
+    void mainTableCheck (AstNode* nodep) {
 	setMode(true/*scoped*/,true/*checking*/, false/*params*/);
 	mainGuts(nodep);
     }
@@ -734,6 +838,10 @@ public:
 	setMode(true/*scoped*/,false/*checking*/, false/*params*/);
 	mainGuts(nodep);
     }
+    void mainCheckTree (AstNode* nodep) {
+	setMode(false/*scoped*/,true/*checking*/, false/*params*/);
+	mainGuts(nodep);
+    }
     void mainParamEmulate (AstNode* nodep) {
 	setMode(false/*scoped*/,false/*checking*/, true/*params*/);
 	mainGuts(nodep);
