@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -69,7 +69,7 @@
 
 class SplitPliVertex : public V3GraphVertex {
 public:
-    SplitPliVertex(V3Graph* graphp)
+    explicit SplitPliVertex(V3Graph* graphp)
 	: V3GraphVertex(graphp) {}
     virtual ~SplitPliVertex() {}
     virtual string name() const { return "*PLI*"; }
