@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -71,7 +71,7 @@ private:
     }
 public:
     // CONSTUCTORS
-    SplitAsFindVisitor(AstAlways* nodep) {
+    explicit SplitAsFindVisitor(AstAlways* nodep) {
 	m_splitVscp = NULL;
 	nodep->accept(*this);
     }
@@ -200,7 +200,7 @@ private:
 
 public:
     // CONSTUCTORS
-    SplitAsVisitor(AstNetlist* nodep) {
+    explicit SplitAsVisitor(AstNetlist* nodep) {
 	m_splitVscp = NULL;
 	AstNode::user1ClearTree();	// user1p() used on entire tree
 	nodep->accept(*this);
