@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2005-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2005-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -37,7 +37,7 @@ public:
     ~V3Double0() {}
 
     // Implicit conversion operators:
-    inline V3Double0 (const vluint64_t v) : m_d(v) { }
+    inline explicit V3Double0 (const vluint64_t v) : m_d(v) { }
     inline operator double () const { return m_d; }
 
     // Explicit operators:
