@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2005-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2005-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -177,7 +177,7 @@ public:
     }
 
     // CONSTRUCTORS
-    StatsReport(ofstream* aofp)
+    explicit StatsReport(ofstream* aofp)
 	: os(*aofp) {
 	header();
 	sumit();
@@ -214,5 +214,5 @@ void V3Stats::statsReport() {
     StatsReport reporter (ofp);
 
     // Cleanup
-    ofp->close(); delete ofp; ofp = NULL;
+    ofp->close(); delete ofp; VL_DANGLING(ofp);
 }
