@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2004-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2004-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -86,7 +86,7 @@ class SubstVarEntry {
 
 public:
     // CONSTRUCTORS
-    SubstVarEntry (AstVar* varp) {	// Construction for when a var is used
+    explicit SubstVarEntry (AstVar* varp) {	// Construction for when a var is used
 	m_varp = varp;
 	m_whole.m_use = false;
 	m_wordAssign = false;
@@ -163,7 +163,7 @@ public:
     }
     void deleteAssign (AstNodeAssign* nodep) {
 	UINFO(5, "Delete "<<nodep<<endl);
-	nodep->unlinkFrBack()->deleteTree(); nodep=NULL;
+	nodep->unlinkFrBack()->deleteTree(); VL_DANGLING(nodep);
     }
     void deleteUnusedAssign() {
 	// If there are unused assignments in this var, kill them
@@ -310,7 +310,7 @@ private:
 	}
 	if (debug()>5)  newp->dumpTree(cout,"       w_new: ");
 	nodep->replaceWith(newp);
-	pushDeletep(nodep);  nodep=NULL;
+	pushDeletep(nodep); VL_DANGLING(nodep);
 	++m_statSubsts;
     }
     virtual void visit(AstWordSel* nodep, AstNUser*) {
@@ -329,7 +329,7 @@ private:
 		// Check that the RHS hasn't changed value since we recorded it.
 		SubstUseVisitor visitor (substp, entryp->getWordStep(word));
 		if (visitor.ok()) {
-		    replaceSubstEtc(nodep, substp); nodep=NULL;
+		    replaceSubstEtc(nodep, substp); VL_DANGLING(nodep);
 		} else {
 		    entryp->consumeWord(word);
 		}
@@ -357,7 +357,7 @@ private:
 		SubstUseVisitor visitor (substp, entryp->getWholeStep());
 		if (visitor.ok()) {
 		    UINFO(8," USEwhole "<<nodep<<endl);
-		    replaceSubstEtc(nodep, substp); nodep=NULL;
+		    replaceSubstEtc(nodep, substp); VL_DANGLING(nodep);
 		} else {
 		    UINFO(8," USEwholeButChg "<<nodep<<endl);
 		    entryp->consumeWhole();
@@ -379,7 +379,7 @@ private:
     }
 public:
     // CONSTUCTORS
-    SubstVisitor(AstNode* nodep) {
+    explicit SubstVisitor(AstNode* nodep) {
 	AstNode::user1ClearTree();	// user1p() used on entire tree
 	AstNode::user2ClearTree();	// user2p() used on entire tree
 	m_ops = 0;
