@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -60,7 +60,7 @@ public:
     virtual void varRefCb(AstVarRef* nodep);	///< Call other-this function on all new var references
 
     // CONSTRUCTORS
-    TableSimulateVisitor(TableVisitor* cbthis) {
+    explicit TableSimulateVisitor(TableVisitor* cbthis) {
 	m_cbthis = cbthis;
     }
     virtual ~TableSimulateVisitor() {}
@@ -328,7 +328,7 @@ private:
 		    setp = new AstConst (outnump->fileline(), *outnump);
 		}
 		// Note InitArray requires us to have the values in inValue order
-		m_tableVarps[outnum]->varp()->valuep()->castInitArray()->addInitsp(setp);
+		m_tableVarps[outnum]->varp()->valuep()->castInitArray()->addValuep(setp);
 		outnum++;
 	    }
 
@@ -336,7 +336,7 @@ private:
 		if (inValue != inValueNextInitArray++)
 		    nodep->v3fatalSrc("InitArray requires us to have the values in inValue order");
 		AstNode* setp = new AstConst (nodep->fileline(), outputChgMask);
-		chgVscp->varp()->valuep()->castInitArray()->addInitsp(setp);
+		chgVscp->varp()->valuep()->castInitArray()->addValuep(setp);
 	    }
 	} // each value
     }
@@ -422,7 +422,7 @@ private:
 	UINFO(4,"  ALWAYS  "<<nodep<<endl);
 	if (treeTest(nodep)) {
 	    // Well, then, I'll be a memory hog.
-	    createTable(nodep); nodep=NULL;
+	    createTable(nodep); VL_DANGLING(nodep);
 	}
     }
     virtual void visit(AstAssignAlias* nodep, AstNUser*) {}
@@ -437,7 +437,7 @@ private:
     }
 public:
     // CONSTRUCTORS
-    TableVisitor(AstNetlist* nodep) {
+    explicit TableVisitor(AstNetlist* nodep) {
 	m_modp = NULL;
 	m_modTables = 0;
 	m_scopep = NULL;
