@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -49,7 +49,7 @@ class TaskBaseVertex : public V3GraphVertex {
     AstNode*	m_impurep;	// Node causing impure function w/ outside references
     bool	m_noInline;	// Marked with pragma
 public:
-    TaskBaseVertex(V3Graph* graphp)
+    explicit TaskBaseVertex(V3Graph* graphp)
 	: V3GraphVertex(graphp), m_impurep(NULL), m_noInline(false) {}
     virtual ~TaskBaseVertex() {}
     bool pure() const { return m_impurep==NULL; }
@@ -79,7 +79,7 @@ public:
 class TaskCodeVertex : public TaskBaseVertex {
     // Top vertex for all calls not under another task
 public:
-    TaskCodeVertex(V3Graph* graphp)
+    explicit TaskCodeVertex(V3Graph* graphp)
 	: TaskBaseVertex(graphp) {}
     virtual ~TaskCodeVertex() {}
     virtual string name() const { return "*CODE*"; }
@@ -182,7 +182,7 @@ private:
     }
     virtual void visit(AstAssignW* nodep, AstNUser*) {
 	m_assignwp = nodep;
-	nodep->iterateChildren(*this); nodep=NULL;  // May delete nodep.
+	nodep->iterateChildren(*this); VL_DANGLING(nodep);  // May delete nodep.
 	m_assignwp = NULL;
     }
     virtual void visit(AstNodeFTaskRef* nodep, AstNUser*) {
@@ -234,7 +234,7 @@ private:
     }
 public:
     // CONSTUCTORS
-    TaskStateVisitor(AstNetlist* nodep) {
+    explicit TaskStateVisitor(AstNetlist* nodep) {
 	m_assignwp = NULL;
 	m_curVxp = new TaskCodeVertex(&m_callGraph);
 	AstNode::user3ClearTree();
@@ -277,7 +277,7 @@ private:
     }
 public:
     // CONSTUCTORS
-    TaskRelinkVisitor(AstBegin* nodep) {  // Passed temporary tree
+    explicit TaskRelinkVisitor(AstBegin* nodep) {  // Passed temporary tree
 	nodep->accept(*this);
     }
     virtual ~TaskRelinkVisitor() {}
@@ -456,7 +456,7 @@ private:
 	{
 	    AstBegin* tempp = new AstBegin(beginp->fileline(),"[EditWrapper]",beginp);
 	    TaskRelinkVisitor visit (tempp);
-	    tempp->stmtsp()->unlinkFrBackWithNext(); tempp->deleteTree(); tempp=NULL;
+	    tempp->stmtsp()->unlinkFrBackWithNext(); tempp->deleteTree(); VL_DANGLING(tempp);
 	}
 	//
 	if (debug()>=9) { beginp->dumpTreeAndNext(cout,"-iotask: "); }
@@ -584,9 +584,14 @@ private:
 	return new AstCStmt(portp->fileline(), stmt);
     }
 
-    AstNode* createAssignInternalToDpi(AstVar* portp, bool isPtr, const string& frSuffix, const string& toSuffix) {
+    AstNode* createAssignInternalToDpi(AstVar* portp, bool isRtn, bool isPtr,
+				       const string& frSuffix, const string& toSuffix) {
 	// Create assignment from internal format into DPI temporary
 	bool bitvec = (portp->basicp()->isBitLogic() && portp->width() > 32);
+	if (isRtn && bitvec) {
+	    portp->v3error("DPI functions cannot return > 32 bits; use a two-state type or task instead: "<<portp->prettyName());
+	    // Code below works, but won't compile right, and IEEE illegal
+	}
 	string stmt;
 	string ket;
 	// Someday we'll have better type support, and this can make variables and casts.
@@ -674,7 +679,7 @@ private:
 	    stmt += "const VerilatedScope* __Vscopep = Verilated::dpiScope();\n";
 	    // If dpiScope is fails and is null; the exportFind function throws and error
 	    string cbtype = v3Global.opt.prefix()+"__Vcb_"+nodep->cname()+"_t";
-	    stmt += cbtype+" __Vcb = ("+cbtype+")__Vscopep->exportFind(__Vfuncnum);\n";
+	    stmt += cbtype+" __Vcb = ("+cbtype+")(VerilatedScope::exportFind(__Vscopep, __Vfuncnum));\n";  // Can't use static_cast
 	    // If __Vcb is null the exportFind function throws and error
 	    dpip->addStmtsp(new AstCStmt(nodep->fileline(), stmt));
 	}
@@ -718,7 +723,7 @@ private:
 	    stmt += "(*__Vcb)(";
 	    args += ");\n";
 	    AstCStmt* newp = new AstCStmt(nodep->fileline(), stmt);
-	    newp->addBodysp(argnodesp); argnodesp=NULL;
+	    newp->addBodysp(argnodesp); VL_DANGLING(argnodesp);
 	    newp->addBodysp(new AstText(nodep->fileline(), args, true));
 	    dpip->addStmtsp(newp);
 	}
@@ -727,14 +732,14 @@ private:
 	for (AstNode* stmtp = nodep->stmtsp(); stmtp; stmtp=stmtp->nextp()) {
 	    if (AstVar* portp = stmtp->castVar()) {
 		if (portp->isIO() && portp->isOutput() && !portp->isFuncReturn()) {
-		    dpip->addStmtsp(createAssignInternalToDpi(portp,true,"__Vcvt",""));
+		    dpip->addStmtsp(createAssignInternalToDpi(portp,false,true,"__Vcvt",""));
 		}
 	    }
 	}
 
 	if (rtnvarp) {
 	    dpip->addStmtsp(createDpiTemp(rtnvarp,""));
-	    dpip->addStmtsp(createAssignInternalToDpi(rtnvarp,false,"__Vcvt",""));
+	    dpip->addStmtsp(createAssignInternalToDpi(rtnvarp,true,false,"__Vcvt",""));
 	    string stmt = "return "+rtnvarp->name()+";\n";
 	    dpip->addStmtsp(new AstCStmt(nodep->fileline(), stmt));
 	}
@@ -784,7 +789,7 @@ private:
 
 		    cfuncp->addStmtsp(createDpiTemp(portp,"__Vcvt"));
 		    if (portp->isInput()) {
-			cfuncp->addStmtsp(createAssignInternalToDpi(portp,false,"","__Vcvt"));
+			cfuncp->addStmtsp(createAssignInternalToDpi(portp,false,false,"","__Vcvt"));
 		    }
 		}
 	    }
@@ -968,10 +973,10 @@ private:
 	{
 	    AstBegin* tempp = new AstBegin(cfuncp->fileline(),"[EditWrapper]",cfuncp);
 	    TaskRelinkVisitor visit (tempp);
-	    tempp->stmtsp()->unlinkFrBackWithNext(); tempp->deleteTree(); tempp=NULL;
+	    tempp->stmtsp()->unlinkFrBackWithNext(); tempp->deleteTree(); VL_DANGLING(tempp);
 	}
 	// Delete rest of cloned task and return new func
-	pushDeletep(nodep); nodep=NULL;
+	pushDeletep(nodep); VL_DANGLING(nodep);
 	if (debug()>=9) { cfuncp->dumpTree(cout,"-userFunc: "); }
 	return cfuncp;
     }
@@ -1069,7 +1074,7 @@ private:
 	    nodep->replaceWith(beginp);
 	}
 	// Cleanup
-	nodep->deleteTree(); nodep=NULL;
+	nodep->deleteTree(); VL_DANGLING(nodep);
 	UINFO(4,"  FTask REF Done.\n");
     }
     virtual void visit(AstNodeFTask* nodep, AstNUser*) {
@@ -1106,7 +1111,7 @@ private:
 		if (AstVar* portp = nodep->fvarp()->castVar()) {
 		    AstVarScope* vscp = m_statep->findVarScope(m_scopep, portp);
 		    UINFO(9,"   funcremovevsc "<<vscp<<endl);
-		    pushDeletep(vscp->unlinkFrBack()); vscp=NULL;
+		    pushDeletep(vscp->unlinkFrBack()); VL_DANGLING(vscp);
 		}
 	    }
 	    for (AstNode* nextp, *stmtp = nodep->stmtsp(); stmtp; stmtp=nextp) {
@@ -1114,13 +1119,13 @@ private:
 		if (AstVar* portp = stmtp->castVar()) {
 		    AstVarScope* vscp = m_statep->findVarScope(m_scopep, portp);
 		    UINFO(9,"   funcremovevsc "<<vscp<<endl);
-		    pushDeletep(vscp->unlinkFrBack()); vscp=NULL;
+		    pushDeletep(vscp->unlinkFrBack()); VL_DANGLING(vscp);
 		}
 	    }
 	    // Just push for deletion, as other references to func may
 	    // remain until visitor exits
 	    nodep->unlinkFrBack();
-	    pushDeletep(nodep); nodep=NULL;
+	    pushDeletep(nodep); VL_DANGLING(nodep);
 	}
 	m_insMode = prevInsMode;
 	m_insStmtp = prevInsStmtp;
@@ -1215,7 +1220,7 @@ V3TaskConnects V3Task::taskConnects(AstNodeFTaskRef* nodep, AstNode* taskStmtsp)
 		pinp->v3error("No such argument '"<<argp->prettyName()
 			      <<"' in function call to "<<nodep->taskp()->prettyTypeName());
 		// We'll just delete it; seems less error prone than making a false argument
-		pinp->unlinkFrBack()->deleteTree(); pinp=NULL;
+		pinp->unlinkFrBack()->deleteTree(); VL_DANGLING(pinp);
 	    } else {
 		if (tconnects[it->second].second) {
 		    pinp->v3error("Duplicate argument '"<<argp->prettyName()
@@ -1234,7 +1239,7 @@ V3TaskConnects V3Task::taskConnects(AstNodeFTaskRef* nodep, AstNode* taskStmtsp)
 		} else {
 		    pinp->v3error("Too many arguments in function call to "<<nodep->taskp()->prettyTypeName());
 		    // We'll just delete it; seems less error prone than making a false argument
-		    pinp->unlinkFrBack()->deleteTree(); pinp=NULL;
+		    pinp->unlinkFrBack()->deleteTree(); VL_DANGLING(pinp);
 		}
 	    } else {
 		tconnects[ppinnum].second = argp;
