@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -40,6 +40,7 @@
 #include "V3Stats.h"
 #include "V3Const.h"
 #include "V3Ast.h"
+#include "V3Simulate.h"
 
 //######################################################################
 // Unroll state, as a visitor of each AstNode
@@ -51,7 +52,6 @@ private:
     AstVarScope*	m_forVscp;		// Iterator variable scope (NULL for generate pass)
     AstConst*		m_varValuep;		// Current value of loop
     AstNode*		m_ignoreIncp;		// Increment node to ignore
-    AstAttrOf*		m_attrp;		// Current attribute
     bool		m_varModeCheck;		// Just checking RHS assignments
     bool		m_varModeReplace;	// Replacing varrefs
     bool		m_varAssignHit;		// Assign var hit
@@ -108,19 +108,12 @@ private:
 	if (precondsp)	UINFO(6, "    Pcon "<<precondsp<<endl);
 	if (condp)	UINFO(6, "    Cond "<<condp<<endl);
 	if (incp)	UINFO(6, "    Inc  "<<incp<<endl);
+
 	// Initial value check
 	AstAssign* initAssp = initp->castAssign();
 	if (!initAssp) return cantUnroll(nodep, "no initial assignment");
 	if (initp->nextp() && initp->nextp()!=nodep) nodep->v3fatalSrc("initial assignment shouldn't be a list");
 	if (!initAssp->lhsp()->castVarRef()) return cantUnroll(nodep, "no initial assignment to simple variable");
-	m_forVarp = initAssp->lhsp()->castVarRef()->varp();
-	m_forVscp = initAssp->lhsp()->castVarRef()->varScopep();
-	if (nodep->castGenFor() && !m_forVarp->isGenVar()) {
-	    nodep->v3error("Non-genvar used in generate for: "<<m_forVarp->prettyName()<<endl);
-	}
-	if (m_generate) V3Const::constifyParamsEdit(initAssp->rhsp());  // rhsp may change
-	AstConst* constInitp = initAssp->rhsp()->castConst();
-	if (!constInitp) return cantUnroll(nodep, "non-constant initializer");
 	//
 	// Condition check
 	if (condp->nextp()) nodep->v3fatalSrc("conditional shouldn't be a list");
@@ -129,93 +122,21 @@ private:
 	AstAssign* incAssp = incp->castAssign();
 	if (!incAssp) return cantUnroll(nodep, "no increment assignment");
 	if (incAssp->nextp()) nodep->v3fatalSrc("increment shouldn't be a list");
-	AstNodeBiop* incInstrp = incAssp->rhsp()->castNodeBiop();
-	//
-	if (m_forVscp) { UINFO(8, "   Loop Variable: "<<m_forVscp<<endl); }
-	else	       { UINFO(8, "   Loop Variable: "<<m_forVarp<<endl); }
-	if (debug()>=9) nodep->dumpTree(cout,"-   for: ");
-	//
-	// Extract the constant loop bounds
-	bool subtract = incInstrp->castSub();
-	{
-	    if (!subtract && !incInstrp->castAdd()) return cantUnroll(nodep, "missing add/sub for incrementer");
-	    AstVarRef* incVarrp   = (subtract ? incInstrp->lhsp()->castVarRef()
-				     : incInstrp->rhsp()->castVarRef());
-	    if (!incVarrp) return cantUnroll(nodep, "missing variable in incrementer");
-	    if (incVarrp->varp() != m_forVarp
-		|| incVarrp->varScopep() != m_forVscp) {
-		return cantUnroll(nodep, "different variables in incrementer");
-	    }
-	}
-	//
-	// Adds have the # on the lhsp because V3Const pushes rhs consts over to the lhs
-	// Subtracts have it on the rhs, because you write i=i-1; i=1-i is non-sensible.
-	AstConst* preconstIncp = (subtract ? incInstrp->rhsp()->castConst()
-				  : incInstrp->lhsp()->castConst());
-	if (m_generate) preconstIncp = V3Const::constifyParamsEdit(preconstIncp)->castConst();
-	AstConst* constIncp = (subtract ? incInstrp->rhsp()->castConst()
-			       : incInstrp->lhsp()->castConst());
-	UINFO(8, "   Inc expr ok:  "<<constIncp<<endl);
-	if (!constIncp) return cantUnroll(nodep, "non-constant increment");
-	if (constIncp->isZero()) return cantUnroll(nodep, "zero increment");  // Or we could loop forever below...
 
-        bool lt  = condp->castLt()  || condp->castLtS();
-        bool lte = condp->castLte() || condp->castLteS();
-	bool gt  = condp->castGt()  || condp->castGtS();
-	bool gte = condp->castGte() || condp->castGteS();
-	if (!lt && !lte && !gt && !gte)
-	    return cantUnroll(nodep, "condition not <=, <, >= or >");
-	AstNodeBiop* cmpInstrp = condp->castNodeBiop();
-	bool cmpVarLhs;
-	if (cmpInstrp->lhsp()->castVarRef()
-	    && cmpInstrp->lhsp()->castVarRef()->varp() == m_forVarp
-	    && cmpInstrp->lhsp()->castVarRef()->varScopep() == m_forVscp) {
-	    cmpVarLhs = true;
-	} else if (cmpInstrp->rhsp()->castVarRef()
-		   && cmpInstrp->rhsp()->castVarRef()->varp() == m_forVarp
-		   && cmpInstrp->rhsp()->castVarRef()->varScopep() == m_forVscp) {
-	    cmpVarLhs = false;
-	} else if (!cmpInstrp->rhsp()->castVarRef()) {
-	    return cantUnroll(nodep, "no variable on rhs of condition");
-	} else {
-	    return cantUnroll(nodep, "different variable in condition");
+	m_forVarp = initAssp->lhsp()->castVarRef()->varp();
+	m_forVscp = initAssp->lhsp()->castVarRef()->varScopep();
+	if (nodep->castGenFor() && !m_forVarp->isGenVar()) {
+	    nodep->v3error("Non-genvar used in generate for: "<<m_forVarp->prettyName()<<endl);
 	}
+	if (m_generate) V3Const::constifyParamsEdit(initAssp->rhsp());  // rhsp may change
 
-	if (m_generate) V3Const::constifyParamsEdit(cmpVarLhs ? cmpInstrp->rhsp()
-						    : cmpInstrp->lhsp());  // rhsp/lhsp may change
-	AstConst* constStopp = (cmpVarLhs ? cmpInstrp->rhsp()->castConst()
-				: cmpInstrp->lhsp()->castConst());
-	if (!constStopp) return cantUnroll(nodep, "non-constant final value");
-	UINFO(8, "   Stop expr ok: "<<constStopp<<endl);
-	//
-	if (constInitp->width()>32 || constInitp->num().isFourState()
-	    || constStopp->width()>32 || constStopp->num().isFourState()
-	    || constIncp->width()>32  || constIncp->num().isFourState())
-	    return cantUnroll(nodep, "init/final/increment too large or four state");
-	vlsint32_t valInit = constInitp->num().toSInt();
-	vlsint32_t valStop = constStopp->num().toSInt();
-	if (gte) valStop++;  if (lte) valStop--;   // 23 >= a, handle as if 24 > a
-	vlsint32_t valInc  = constIncp->num().toSInt();
-	if (subtract) valInc = -valInc;
-	UINFO(8,"     In Numbers: for (v="<<valInit<<"; v<"<<valStop<<"; v=v+"<<valInc<<")\n");
-	//
-	if (!m_generate) {
-	    int loops = ((valStop - valInit)/valInc);
-	    if (loops < 0) { loops += (1ULL<<constStopp->width()); } // Will roll around
-	    UINFO(8, "         ~Iters: "<<loops<<" c="<<unrollCount()<<endl);
-	    if (loops > unrollCount())
-		return cantUnroll(nodep, "too many iterations");
+	// This check shouldn't be needed when using V3Simulate
+	// however, for repeat loops, the loop variable is auto-generated
+	// and the initp statements will reference a variable outside of the initp scope
+	// alas, failing to simulate.
+	AstConst* constInitp = initAssp->rhsp()->castConst();
+	if (!constInitp) return cantUnroll(nodep, "non-constant initializer");
 
-	    // Less than 10 statements in the body?
-	    int bodySize = 0;
-	    int bodyLimit = v3Global.opt.unrollStmts();
-	    if (loops>0) bodyLimit = v3Global.opt.unrollStmts() / loops;
-	    if (bodySizeOverRecurse(precondsp, bodySize/*ref*/, bodyLimit)
-		|| bodySizeOverRecurse(bodysp, bodySize/*ref*/, bodyLimit)
-		|| bodySizeOverRecurse(incp, bodySize/*ref*/, bodyLimit)) {
-		return cantUnroll(nodep, "too many statements");
-	    }
-	}
 	//
 	// Now, make sure there's no assignment to this variable in the loop
 	m_varModeCheck = true;
@@ -227,26 +148,131 @@ private:
 	m_varModeCheck = false;
 	m_ignoreIncp = NULL;
 	if (m_varAssignHit) return cantUnroll(nodep, "genvar assigned *inside* loop");
+
 	//
+	if (m_forVscp) { UINFO(8, "   Loop Variable: "<<m_forVscp<<endl); }
+	else	       { UINFO(8, "   Loop Variable: "<<m_forVarp<<endl); }
+	if (debug()>=9) nodep->dumpTree(cout,"-   for: ");
+
+
+	if (!m_generate) {
+	    AstAssign *incpAssign = incp->castAssign();
+	    if (!canSimulate(incpAssign->rhsp())) return cantUnroll(incp, "Unable to simulate increment");
+	    if (!canSimulate(condp)) return cantUnroll(condp, "Unable to simulate condition");
+
+	    // Check whether to we actually want to try and unroll.
+	    int loops;
+	    if (!countLoops(initAssp, condp, incp, unrollCount(), loops))
+		return cantUnroll(nodep, "Unable to simulate loop");
+
+	    // Less than 10 statements in the body?
+	    int bodySize = 0;
+	    int bodyLimit = v3Global.opt.unrollStmts();
+	    if (loops>0) bodyLimit = v3Global.opt.unrollStmts() / loops;
+	    if (bodySizeOverRecurse(precondsp, bodySize/*ref*/, bodyLimit)
+		|| bodySizeOverRecurse(bodysp, bodySize/*ref*/, bodyLimit)
+		|| bodySizeOverRecurse(incp, bodySize/*ref*/, bodyLimit)) {
+		return cantUnroll(nodep, "too many statements");
+	    }
+	}
 	// Finally, we can do it
-	forUnroller(nodep, initp, precondsp, incp, bodysp,
-		    constInitp->num(),
-		    cmpInstrp, constStopp->num(), cmpVarLhs,
-		    incInstrp, constIncp->num()); nodep = NULL;
+	if (!forUnroller(nodep, initAssp, condp, precondsp, incp, bodysp)) {
+	    return cantUnroll(nodep, "Unable to unroll loop");
+	}
+	VL_DANGLING(nodep);
 	// Cleanup
 	return true;
     }
 
-    void forUnroller(AstNode* nodep,
-		     AstNode* initp,
+    bool canSimulate(AstNode *nodep) {
+	SimulateVisitor simvis;
+	AstNode* clone = nodep->cloneTree(true);
+	simvis.mainCheckTree(clone);
+	return simvis.optimizable();
+    }
+
+    bool simulateTree(AstNode *nodep, const V3Number *loopValue, AstNode *dtypep, V3Number &outNum) {
+	AstNode* clone = nodep->cloneTree(true);
+	if (!clone) {
+	    nodep->v3fatalSrc("Failed to clone tree");
+	    return false;
+	}
+	if (loopValue) {
+	    m_varValuep = new AstConst (nodep->fileline(), *loopValue);
+	    // Iteration requires a back, so put under temporary node
+	    AstBegin* tempp = new AstBegin (nodep->fileline(), "[EditWrapper]", clone);
+	    m_varModeReplace = true;
+	    tempp->stmtsp()->iterateAndNext(*this);
+	    m_varModeReplace = false;
+	    clone = tempp->stmtsp()->unlinkFrBackWithNext();
+	    tempp->deleteTree();
+	    tempp = NULL;
+	    pushDeletep(m_varValuep); m_varValuep = NULL;
+	}
+	SimulateVisitor simvis;
+	simvis.mainParamEmulate(clone);
+	if (!simvis.optimizable()) {
+	    UINFO(3, "Unable to simulate" << endl);
+	    if (debug()>=9) nodep->dumpTree(cout,"- _simtree: ");
+	    return false;
+	}
+	// Fetch the result
+	V3Number* res = simvis.fetchNumberNull(clone);
+	if (!res) {
+	    UINFO(3, "No number returned from simulation" << endl);
+	    return false;
+	}
+	// Patch up datatype
+	if (dtypep) {
+	    AstConst new_con (clone->fileline(), *res);
+	    new_con.dtypeFrom(dtypep);
+	    outNum = new_con.num();
+	    return true;
+	}
+	outNum = *res;
+	return true;
+    }
+
+    bool countLoops(AstAssign *initp, AstNode *condp, AstNode *incp, int max, int &outLoopsr) {
+	outLoopsr = 0;
+	V3Number loopValue = V3Number(initp->fileline());
+	if (!simulateTree(initp->rhsp(), NULL, initp, loopValue)) {
+	    return false;
+	}
+	while (1) {
+	    V3Number res = V3Number(initp->fileline());
+	    if (!simulateTree(condp, &loopValue, NULL, res)) {
+		return false;
+	    }
+	    if (!res.isEqOne()) {
+		break;
+	    }
+
+	    outLoopsr++;
+
+	    // Run inc
+	    AstAssign* incpass = incp->castAssign();
+	    V3Number newLoopValue = V3Number(initp->fileline());
+	    if (!simulateTree(incpass->rhsp(), &loopValue, incpass, newLoopValue)) {
+		return false;
+	    }
+	    loopValue.opAssign(newLoopValue);
+	    if (outLoopsr > max) {
+		return false;
+	    }
+	}
+	return true;
+    }
+
+    bool forUnroller(AstNode* nodep,
+		     AstAssign* initp,
+		     AstNode* condp,
 		     AstNode* precondsp,
-		     AstNode* incp, AstNode* bodysp,
-		     const V3Number& numInit,
-		     AstNodeBiop* cmpInstrp, const V3Number& numStop, bool cmpVarLhs,
-		     AstNodeBiop* incInstrp, const V3Number& numInc) {
-	UINFO(4, "   Unroll for var="<<numInit<<"; var<"<<numStop<<"; var+="<<numInc<<endl);
-	UINFO(6, "    cmpI "<<cmpInstrp<<endl);
-	UINFO(6, "    IncI "<<incInstrp<<endl);
+		     AstNode* incp, AstNode* bodysp) {
+	V3Number loopValue = V3Number(nodep->fileline());
+	if (!simulateTree(initp->rhsp(), NULL, initp, loopValue)) {
+	    return false;
+	}
 	AstNode* stmtsp = NULL;
 	if (initp) {
 	    initp->unlinkFrBack();	// Always a single statement; nextp() may be nodep
@@ -254,42 +280,34 @@ private:
 	}
 	if (precondsp) {
 	    precondsp->unlinkFrBackWithNext();
-	    // cppcheck-suppress nullPointer  // addNextNull deals with it
-	    stmtsp = stmtsp->addNextNull(precondsp);
+	    stmtsp = AstNode::addNextNull(stmtsp, precondsp);
 	}
 	if (bodysp) {
 	    bodysp->unlinkFrBackWithNext();
-	    // cppcheck-suppress nullPointer  // addNextNull deals with it
-	    stmtsp = stmtsp->addNextNull(bodysp);  // Maybe null if no body
+	    stmtsp = AstNode::addNextNull(stmtsp, bodysp);  // Maybe null if no body
 	}
 	if (incp && !nodep->castGenFor()) {  // Generates don't need to increment loop index
 	    incp->unlinkFrBackWithNext();
-	    // cppcheck-suppress nullPointer  // addNextNull deals with it
-	    stmtsp = stmtsp->addNextNull(incp);  // Maybe null if no body
+	    stmtsp = AstNode::addNextNull(stmtsp, incp);  // Maybe null if no body
 	}
 	// Mark variable to disable some later warnings
 	m_forVarp->usedLoopIdx(true);
 
-	// If it's a While, then incp is already part of bodysp.
-	V3Number loopValue(nodep->fileline(), m_forVarp->width());  // May differ in size from numInitp
-	loopValue.opAssign(numInit);
-
 	AstNode* newbodysp = NULL;
 	++m_statLoops;
 	if (stmtsp) {
 	    int times = 0;
 	    while (1) {
 		UINFO(8,"      Looping "<<loopValue<<endl);
-		// if loopValue<valStop
-		V3Number contin (nodep->fileline(), 1);
-		if (cmpVarLhs) {
-		    cmpInstrp->numberOperate(contin, loopValue, numStop);
-		} else {
-		    cmpInstrp->numberOperate(contin, numStop, loopValue);
+		V3Number res = V3Number(nodep->fileline());
+		if (!simulateTree(condp, &loopValue, NULL, res)) {
+		    nodep->v3error("Loop unrolling failed.");
+		    return false;
 		}
-		if (contin.isEqZero()) {
+		if (!res.isEqOne()) {
 		    break;  // Done with the loop
-		} else {
+		}
+		else {
 		    // Replace iterator values with constant.
 		    AstNode* oneloopp = stmtsp->cloneTree(true);
 
@@ -301,14 +319,14 @@ private:
 			m_varModeReplace = true;
 			tempp->stmtsp()->iterateAndNext(*this);
 			m_varModeReplace = false;
-			oneloopp = tempp->stmtsp()->unlinkFrBackWithNext(); tempp->deleteTree(); tempp=NULL;
+			oneloopp = tempp->stmtsp()->unlinkFrBackWithNext(); tempp->deleteTree(); VL_DANGLING(tempp);
 		    }
 		    if (m_generate) {
 			string index = AstNode::encodeNumber(m_varValuep->toSInt());
 			string nname = m_beginName + "__BRA__" + index + "__KET__";
 			oneloopp = new AstBegin(oneloopp->fileline(),nname,oneloopp,true);
 		    }
-
+		    pushDeletep(m_varValuep); m_varValuep=NULL;
 		    if (newbodysp) newbodysp->addNext(oneloopp);
 		    else newbodysp = oneloopp;
 
@@ -318,23 +336,26 @@ private:
 			break;
 		    }
 
-		    //loopValue += valInc
-		    V3Number newnum(nodep->fileline(), m_forVarp->width());  // Can't increment in-place
-		    incInstrp->numberOperate(newnum, loopValue, numInc);
-		    loopValue.opAssign(newnum);
-
-		    pushDeletep(m_varValuep); m_varValuep=NULL;
+		    // loopValue += valInc
+		    AstAssign *incpass = incp->castAssign();
+		    V3Number newLoopValue = V3Number(nodep->fileline());
+		    if (!simulateTree(incpass->rhsp(), &loopValue, incpass, newLoopValue)) {
+			nodep->v3error("Loop unrolling failed");
+			return false;
+		    }
+		    loopValue.opAssign(newLoopValue);
 		}
 	    }
 	}
 	// Replace the FOR()
 	if (newbodysp) nodep->replaceWith(newbodysp);
 	else nodep->unlinkFrBack();
-	if (bodysp) { pushDeletep(bodysp); bodysp=NULL; }
-	if (precondsp) { pushDeletep(precondsp); precondsp=NULL; }
-	if (initp) { pushDeletep(initp); initp=NULL; }
-	if (incp && !incp->backp()) { pushDeletep(incp); incp=NULL; }
+	if (bodysp) { pushDeletep(bodysp); VL_DANGLING(bodysp); }
+	if (precondsp) { pushDeletep(precondsp); VL_DANGLING(precondsp); }
+	if (initp) { pushDeletep(initp); VL_DANGLING(initp); }
+	if (incp && !incp->backp()) { pushDeletep(incp); VL_DANGLING(incp); }
 	if (debug()>=9) newbodysp->dumpTree(cout,"-  _new: ");
+	return true;
     }
 
     virtual void visit(AstWhile* nodep, AstNUser*) {
@@ -343,11 +364,11 @@ private:
 	} else {
 	    // Constify before unroll call, as it may change what is underneath.
 	    if (nodep->precondsp()) V3Const::constifyEdit(nodep->precondsp());  // precondsp may change
-	    if (nodep->condp()) V3Const::constifyEdit(nodep->condp()); //condp may change
+	    if (nodep->condp()) V3Const::constifyEdit(nodep->condp()); // condp may change
 	    // Grab initial value
 	    AstNode* initp = NULL;  // Should be statement before the while.
 	    if (nodep->backp()->nextp() == nodep) initp=nodep->backp();
-	    if (initp) { V3Const::constifyEdit(initp); initp=NULL; }
+	    if (initp) { V3Const::constifyEdit(initp); VL_DANGLING(initp); }
 	    if (nodep->backp()->nextp() == nodep) initp=nodep->backp();
 	    // Grab assignment
 	    AstNode* incp = NULL;  // Should be last statement
@@ -355,14 +376,14 @@ private:
 	    if (nodep->incsp()) incp = nodep->incsp();
 	    else {
 		for (incp = nodep->bodysp(); incp && incp->nextp(); incp = incp->nextp()) {}
-		if (incp) { V3Const::constifyEdit(incp); incp=NULL; }
+		if (incp) { V3Const::constifyEdit(incp); VL_DANGLING(incp); }
 		for (incp = nodep->bodysp(); incp && incp->nextp(); incp = incp->nextp()) {}  // Again, as may have changed
 	    }
 	    // And check it
 	    if (forUnrollCheck(nodep, initp,
 			       nodep->precondsp(), nodep->condp(),
 			       incp, nodep->bodysp())) {
-		pushDeletep(nodep); nodep=NULL; // Did replacement
+		pushDeletep(nodep); VL_DANGLING(nodep); // Did replacement
 	    }
 	}
     }
@@ -384,11 +405,11 @@ private:
 		// we'd need to initialize the variable to the initial
 		// condition, but they'll become while's which can be
 		// deleted by V3Const.
-		nodep->unlinkFrBack()->deleteTree(); nodep=NULL;
+		nodep->unlinkFrBack()->deleteTree(); VL_DANGLING(nodep);
 	    } else if (forUnrollCheck(nodep, nodep->initsp(),
 				      NULL, nodep->condp(),
 				      nodep->incsp(), nodep->bodysp())) {
-		pushDeletep(nodep); nodep=NULL; // Did replacement
+		pushDeletep(nodep); VL_DANGLING(nodep); // Did replacement
 	    } else {
 		nodep->v3error("For loop doesn't have genvar index, or is malformed");
 	    }
@@ -401,12 +422,6 @@ private:
 	    nodep->v3error("V3Begin should have removed standard FORs");
 	}
     }
-    virtual void visit(AstAttrOf* nodep, AstNUser*) {
-	AstAttrOf* oldAttr = m_attrp;
-	m_attrp = nodep;
-	nodep->iterateChildren(*this);
-	m_attrp = oldAttr;
-    }
 
     virtual void visit(AstVarRef* nodep, AstNUser*) {
 	if (m_varModeCheck
@@ -416,11 +431,11 @@ private:
 	    UINFO(8,"   Itervar assigned to: "<<nodep<<endl);
 	    m_varAssignHit = true;
 	}
+
 	if (m_varModeReplace
 	    && nodep->varp() == m_forVarp
 	    && nodep->varScopep() == m_forVscp
-	    && !nodep->lvalue()
-	    && !m_attrp) {  // Most likely under a select
+	    && !nodep->lvalue()) {
 	    AstNode* newconstp = m_varValuep->cloneTree(false);
 	    nodep->replaceWith(newconstp);
 	    pushDeletep(nodep);
@@ -447,13 +462,12 @@ public:
 	m_varModeReplace = false;
 	m_generate = generate;
 	m_beginName = beginName;
-	m_attrp = NULL;
 	//
 	nodep->accept(*this);
     }
     virtual ~UnrollVisitor() {
-	V3Stats::addStat("Optimizations, Unrolled Loops", m_statLoops);
-	V3Stats::addStat("Optimizations, Unrolled Iterations", m_statIters);
+	V3Stats::addStatSum("Optimizations, Unrolled Loops", m_statLoops);
+	V3Stats::addStatSum("Optimizations, Unrolled Iterations", m_statIters);
     }
 };
 
