@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2015 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -32,15 +32,16 @@
 //######################################################################
 
 /// Remove all $signed, $unsigned, we're done with them.
+/// This step is only called on real V3Width, not intermediate e.g. widthParams
 
 class WidthRemoveVisitor : public AstNVisitor {
 private:
     // VISITORS
     virtual void visit(AstSigned* nodep, AstNUser*) {
-	replaceWithSignedVersion(nodep, nodep->lhsp()->unlinkFrBack()); nodep=NULL;
+	replaceWithSignedVersion(nodep, nodep->lhsp()->unlinkFrBack()); VL_DANGLING(nodep);
     }
     virtual void visit(AstUnsigned* nodep, AstNUser*) {
-	replaceWithSignedVersion(nodep, nodep->lhsp()->unlinkFrBack()); nodep=NULL;
+	replaceWithSignedVersion(nodep, nodep->lhsp()->unlinkFrBack()); VL_DANGLING(nodep);
     }
     virtual void visit(AstNode* nodep, AstNUser*) {
 	nodep->iterateChildren(*this);
@@ -49,14 +50,14 @@ private:
 	UINFO(6," Replace "<<nodep<<" w/ "<<newp<<endl);
 	nodep->replaceWith(newp);
 	newp->dtypeFrom(nodep);
-	pushDeletep(nodep); nodep=NULL;
+	pushDeletep(nodep); VL_DANGLING(nodep);
     }
 public:
     // CONSTRUCTORS
     WidthRemoveVisitor() {}
     virtual ~WidthRemoveVisitor() {}
     AstNode* mainAcceptEdit(AstNode* nodep) {
-	return nodep->acceptSubtreeReturnEdits(*this);
+	return nodep->iterateSubtreeReturnEdits(*this);
     }
 };
 
@@ -119,7 +120,7 @@ private:
 	    AstNode* oldp = nodep; nodep = newp;
 	    //if (debug()>4) oldp->dumpTree(cout,"  fixConstSize_old: ");
 	    //if (debug()>4) newp->dumpTree(cout,"              _new: ");
-	    pushDeletep(oldp); oldp=NULL;
+	    pushDeletep(oldp); VL_DANGLING(oldp);
 	}
 	editDType(nodep);
     }
@@ -127,9 +128,17 @@ private:
 	visitIterateNodeDType(nodep);
     }
     virtual void visit(AstNodeClassDType* nodep, AstNUser*) {
+	if (nodep->user1SetOnce()) return;  // Process once
 	visitIterateNodeDType(nodep);
 	nodep->clearCache();
     }
+    virtual void visit(AstParamTypeDType* nodep, AstNUser*) {
+	if (nodep->user1SetOnce()) return;  // Process once
+	visitIterateNodeDType(nodep);
+	// Move to type table as all dtype pointers must resolve there
+	nodep->unlinkFrBack();  // Make non-child
+	v3Global.rootp()->typeTablep()->addTypesp(nodep);
+    }
     void visitIterateNodeDType(AstNodeDType* nodep) {
 	// Rather than use dtypeChg which may make new nodes, we simply edit in place,
 	// as we don't need to preserve any widthMin's, and every dtype with the same width
@@ -151,7 +160,7 @@ private:
     }
 public:
     // CONSTUCTORS
-    WidthCommitVisitor(AstNetlist* nodep) {
+    explicit WidthCommitVisitor(AstNetlist* nodep) {
 	// Were changing widthMin's, so the table is now somewhat trashed
 	nodep->typeTablep()->clearCache();
 	nodep->accept(*this);
