@@ -14,6 +14,7 @@
 //*************************************************************************
 
 #include "vpi_user.h"
+#include <sstream>
 
 class TestSimulator {
 private:
@@ -69,9 +70,11 @@ public:
     }
     // return absolute scope of obj
     static const char* rooted(const char *obj) {
-	static char buf[256];
-	snprintf(buf, sizeof(buf), "%s.%s", top(), obj);
-	return buf;
+	static string buf;
+	ostringstream os;
+	os<<top()<<"."<<obj;
+	buf = os.str();
+	return buf.c_str();
     }
 };
 
