@@ -1,6 +1,6 @@
 // DESCRIPTION: Verilator Test: Top level main for invoking model
 //
-// Copyright 2003-2015 by Wilson Snyder. This program is free software; you can
+// Copyright 2003-2016 by Wilson Snyder. This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -29,7 +29,7 @@ int main(int argc, char **argv, char **env) {
 #define CYCTIME 10
 
     // Cycle the interpreter
-    while (main_time < CYCTIME*top->v->CYCLES) {
+    while (main_time < CYCTIME*top->vgen->CYCLES) {
 	top->eval();
 	main_time += CYCTIME/2;
 	top->clk = !top->clk;
