@@ -196,7 +196,7 @@ foreach my $op (keys %ops2) {
 
 
 my  $opt_seed=5;
-our $Opt_NumOps = 300;
+our $Opt_NumOps = 30;
 our $Opt_Depth = 4;
 our $Opt_Signed = 1;
 our $Opt_Raise;
@@ -377,17 +377,15 @@ sub rnd_const {
     my $val = Bit::Vector->new($width);
     if ($v<25) {	# zero
     } elsif ($v<50) {	# ones
-	$val->Word_Store(0,~0);
-	$val->Word_Store(1,~0) if $width>32;
-	$val->Word_Store(2,~0) if $width>64;
-	$val->Word_Store(3,~0) if $width>96;
+	for (my $w=0; $w<$val->Word_Size; ++$w) {
+	    $val->Word_Store(0,~0);
+	}
     } elsif ($v<60) {	# one
 	$val->Word_Store(0,1);
     } else { #random
-	$val->Word_Store(0,rnd_int());
-	$val->Word_Store(1,rnd_int()) if $width>32;
-	$val->Word_Store(2,rnd_int()) if $width>64;
-	$val->Word_Store(3,rnd_int()) if $width>96;
+	for (my $w=0; $w<$val->Word_Size; ++$w) {
+	    $val->Word_Store($w,rnd_int());
+	}
     }
     $treeref->{val} = $val;
 }
@@ -1136,7 +1134,7 @@ Include some signed arithmetic in the generated code.  Experimental.
 
 =head1 DISTRIBUTION
 
-Copyright 2001-2015 by Wilson Snyder.  Verilator is free software; you can
+Copyright 2001-2016 by Wilson Snyder.  Verilator is free software; you can
 redistribute it and/or modify it under the terms of either the GNU Lesser
 General Public License Version 3 or the Perl Artistic License Version 2.0.
 
